/**
 * @license Copyright (c) Microsoft Corporation. All rights reserved.
 */
var MscrmControls;
(function (MscrmControls) {
    var AppCommon;
    (function (AppCommon) {
        var ExportToExcel;
        (function (ExportToExcel) {
            'use strict';
            /**
             * Constants used in the control
             */
            var Constants = (function () {
                function Constants() {
                }
                return Constants;
            }());
            /// <summary>
            /// Empty Guid String
            /// </summary>
            Constants.EmptyGuid = "{00000000-0000-0000-0000-000000000000}";
            /// <summary>
            /// Empty Guid in String format
            /// </summary>
            Constants.EmptyGuidFormatted = "00000000-0000-0000-0000-000000000000";
            // Event Names
            // resx names
            Constants.EnterKeyValue = "Enter"; // Keyboard Enter
            Constants.TabKeyCode = 9; // Tab KeyCode
            Constants.EnterKeyCode = 13; // Enter KeyCode
            Constants.EscapeKeyCode = 27; // Escape KeyCode
            Constants.SpaceKeyCode = 32; // Space KeyCode
            Constants.UpArrowKeyCode = 38; // Keyboard Up Arrow
            Constants.DownArrowKeyCode = 40; // Keyboard Down Arrow
            Constants.Letter_E_KeyCode = 69; // Letter E KeyCode
            Constants.EnterKey = 13; // Keyboard Down Arrow
            Constants.TabKey = 9; // Keyboard E character
            // Values for AccessibilityText
            Constants.ArticleActionToNameMapping = {
                0: "LinkArticle",
                1: "UnlinkArticle",
                2: "CopyLink",
                3: "EmailArticle",
                5: "EmailArticleContent",
                6: "OpenResult",
                7: "SelectedResult"
            };
            Constants.ArticleActionToMicrosoftIconSymbolMapping = {
                0: 74,
                1: 75,
                2: 76,
                3: 77,
                5: 89
            };
            /**
            * State code mapping for Knowledge Article entity
            */
            Constants.ArticleState = {
                Draft: "0",
                Approved: "1",
                Published: "3"
            };
            Constants.SearchWidgetEventMapping = {
                0: "SearchPerformed",
                1: "LanguageFilterUsed",
                2: "ArticleStateFilterUsed",
                3: "SortChanged",
                4: "FirstPageFirstResultSelected",
                5: "FirstPageOtherResultSelected",
                6: "OtherPageOtherResultSelected",
                7: "BackButtonClickedWithoutAction",
                8: "ShowMoreClicked",
                9: "LinkButtonClicked",
                10: "LinkAndEmailButtonClicked",
                11: "LinkAndEmailContentButtonClicked",
                12: "CopyLinkButtonClicked"
            };
            Constants.EntitiesXml = '<?xml version=\"1.0\" encoding=\"utf-16\" standalone=\"yes\"?><entities><entity name=\"account\" entitydisplayname=\"Account\" displayname=\"Account\" alias=\"\" relid=\"70816501edb94740a16c6a5efbc05d84\" /><entity name=\"lead\" entitydisplayname=\"Lead\" displayname=\"Originating Lead (Lead)\" griddisplaynamemask=\"{0} (Originating Lead)\" relid=\"b3e3c88a460ce81180e500155d0a3002\" fromattr=\"leadid\" toattr=\"originatingleadid\" alias=\"lead\" /><entity name=\"pricelevel\" entitydisplayname=\"Price List\" displayname=\"Price List (Price List)\" griddisplaynamemask=\"{0} (Price List)\" relid=\"6a0fdfc1460ce81180e500155d0a3002\" fromattr=\"pricelevelid\" toattr=\"defaultpricelevelid\" alias=\"pricelevel\" /><entity name=\"equipment\" entitydisplayname=\"Facility/Equipment\" displayname=\"Preferred Facility/Equipment (Facility/Equipment)\" griddisplaynamemask=\"{0} (Preferred Facility/Equipment)\" relid=\"313fb0bf470ce81180e500155d0a3002\" fromattr=\"equipmentid\" toattr=\"preferredequipmentid\" alias=\"equipment\" /><entity name=\"service\" entitydisplayname=\"Service\" displayname=\"Preferred Service (Service)\" griddisplaynamemask=\"{0} (Preferred Service)\" relid=\"1440b0bf470ce81180e500155d0a3002\" fromattr=\"serviceid\" toattr=\"preferredserviceid\" alias=\"service\" /><entity name=\"territory\" entitydisplayname=\"Territory\" displayname=\"Territory (Territory)\" griddisplaynamemask=\"{0} (Territory)\" relid=\"7f1d32b9480ce81180e500155d0a3002\" fromattr=\"territoryid\" toattr=\"territoryid\" alias=\"territory\" /><entity name=\"contact\" entitydisplayname=\"Contact\" displayname=\"Primary Contact (Contact)\" griddisplaynamemask=\"{0} (Primary Contact)\" relid=\"410707b195544cd984376608b1802904\" fromattr=\"contactid\" toattr=\"primarycontactid\" relationshipid=\"{FAA72A46-7632-4B9A-942A-751D6F401CB5}\" relationshipname=\"account_primary_contact\" alias=\"accountprimarycontactidcontactcontactid\" /><entity name=\"systemuser\" entitydisplayname=\"User\" displayname=\"Preferred User (User)\" griddisplaynamemask=\"{0} (Preferred User)\" relid=\"a6b48e23fada4b7f8655530bba050765\" fromattr=\"systemuserid\" toattr=\"preferredsystemuserid\" alias=\"systemuser\" /><entity name=\"externalparty\" entitydisplayname=\"External Party\" displayname=\"Created By (External Party) (External Party)\" griddisplaynamemask=\"{0} (Created By (External Party))\" relid=\"9967fe7d84ee4a269ad7a8fdbdfa2316\" fromattr=\"externalpartyid\" toattr=\"createdbyexternalparty\" alias=\"externalparty\" /><entity name=\"systemuser\" entitydisplayname=\"User\" displayname=\"Modified By (User)\" griddisplaynamemask=\"{0} (Modified By)\" relid=\"8be02a9d07764c76b35f1c92dd791d9e\" fromattr=\"systemuserid\" toattr=\"modifiedby\" alias=\"systemuser\" /><entity name=\"account\" entitydisplayname=\"Account\" displayname=\"Parent Account (Account)\" griddisplaynamemask=\"{0} (Parent Account)\" relid=\"57511732b5534cfbbcf2d280f9f8c6f1\" fromattr=\"accountid\" toattr=\"parentaccountid\" alias=\"account\" /><entity name=\"businessunit\" entitydisplayname=\"Business Unit\" displayname=\"Owning Business Unit (Business Unit)\" griddisplaynamemask=\"{0} (Owning Business Unit)\" relid=\"6ad8133d2f1e4c43a3da460bacb3d6a5\" fromattr=\"businessunitid\" toattr=\"owningbusinessunit\" alias=\"businessunit\" /><entity name=\"transactioncurrency\" entitydisplayname=\"Currency\" displayname=\"Currency (Currency)\" griddisplaynamemask=\"{0} (Currency)\" relid=\"1742bd269b6b4100becf4a5ea2f0d819\" fromattr=\"transactioncurrencyid\" toattr=\"transactioncurrencyid\" alias=\"transactioncurrency\" /><entity name=\"systemuser\" entitydisplayname=\"User\" displayname=\"Owning User (User)\" griddisplaynamemask=\"{0} (Owning User)\" relid=\"5e98d04200bb494ab231eadc8d4a94ec\" fromattr=\"systemuserid\" toattr=\"owninguser\" alias=\"systemuser\" /><entity name=\"systemuser\" entitydisplayname=\"User\" displayname=\"Created By (Delegate) (User)\" griddisplaynamemask=\"{0} (Created By (Delegate))\" relid=\"b662096051ad4586b2845777f4b77ce2\" fromattr=\"systemuserid\" toattr=\"createdonbehalfby\" alias=\"systemuser\" /><entity name=\"processstage\" entitydisplayname=\"Process Stage\" displayname=\"Process Stage (Process Stage)\" griddisplaynamemask=\"{0} (Process Stage)\" relid=\"b74b70621d03436eb4def90ed78d5412\" fromattr=\"processstageid\" toattr=\"stageid\" alias=\"processstage\" /><entity name=\"sla\" entitydisplayname=\"SLA\" displayname=\"SLA (SLA)\" griddisplaynamemask=\"{0} (SLA)\" relid=\"0d17829c506548b8b94b11105febcd15\" fromattr=\"slaid\" toattr=\"slaid\" alias=\"sla\" /><entity name=\"systemuser\" entitydisplayname=\"User\" displayname=\"Created By (User)\" griddisplaynamemask=\"{0} (Created By)\" relid=\"54ec56504a4f4da1b7820a4a8e1e3e84\" fromattr=\"systemuserid\" toattr=\"createdby\" alias=\"systemuser\" /><entity name=\"externalparty\" entitydisplayname=\"External Party\" displayname=\"Modified By (External Party) (External Party)\" griddisplaynamemask=\"{0} (Modified By (External Party))\" relid=\"4f4276b73d184fb199b805c833c81a3e\" fromattr=\"externalpartyid\" toattr=\"modifiedbyexternalparty\" alias=\"externalparty\" /><entity name=\"sla\" entitydisplayname=\"SLA\" displayname=\"Last SLA applied (SLA)\" griddisplaynamemask=\"{0} (Last SLA applied)\" relid=\"12140b38c01942ad83498b939b839939\" fromattr=\"slaid\" toattr=\"slainvokedid\" alias=\"sla\" /><entity name=\"systemuser\" entitydisplayname=\"User\" displayname=\"Modified By (Delegate) (User)\" griddisplaynamemask=\"{0} (Modified By (Delegate))\" relid=\"2074fc1d84a248aca47cfcf1d249a052\" fromattr=\"systemuserid\" toattr=\"modifiedonbehalfby\" alias=\"systemuser\" /><entity name=\"team\" entitydisplayname=\"Team\" displayname=\"Owning Team (Team)\" griddisplaynamemask=\"{0} (Owning Team)\" relid=\"55aca45612e145d8989edce78f9a31b8\" fromattr=\"teamid\" toattr=\"owningteam\" alias=\"team\" /></entities>';
            Constants.FieldAndPropertiesXml = '<?xml version="1.0" encoding="utf-16"?><fieldpropertiesxml><propertiesxml><entities><entity name="account" objecttypecode="1"><fields><field name="address2_line1" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address 2: Street 1" languagecode="1033" /></displaynames></field><field name="territorycode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Territory Code" languagecode="1033" /></displaynames></field><field name="accountid" requiredlevel="systemrequired"><displaynames><displayname description="Account" languagecode="1033" /></displaynames></field><field name="address1_telephone1" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address Phone" languagecode="1033" /></displaynames></field><field name="address1_postofficebox" requiredlevel="none" imemode="inactive" format="text" maxlength="20"><displaynames><displayname description="Address 1: Post Office Box" languagecode="1033" /></displaynames></field><field name="openrevenue" requiredlevel="none" issecured="true" sourcetype="2" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Open Revenue" languagecode="1033" /></displaynames></field><field name="marketcap" requiredlevel="none" imemode="disabled" minvalue="0" maxvalue="100000000000000" accuracy="2"><displaynames><displayname description="Market Capitalization" languagecode="1033" /></displaynames></field><field name="address1_name" requiredlevel="none" imemode="active" format="text" maxlength="200"><displaynames><displayname description="Address 1: Name" languagecode="1033" /></displaynames></field><field name="emailaddress3" requiredlevel="none" imemode="inactive" format="email" maxlength="100"><displaynames><displayname description="Email Address 3" languagecode="1033" /></displaynames></field><field name="address1_telephone3" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 1: Telephone 3" languagecode="1033" /></displaynames></field><field name="address2_county" requiredlevel="none" imemode="active" format="text" maxlength="50"><displaynames><displayname description="Address 2: County" languagecode="1033" /></displaynames></field><field name="opendeals_date" requiredlevel="none" imemode="inactive" format="datetime" Behavior="Dynamic"><displaynames><displayname description="Open Deals (Last Updated On)" languagecode="1033" /></displaynames></field><field name="numberofemployees" requiredlevel="none" imemode="disabled" format="" minvalue="0" maxvalue="1000000000"><displaynames><displayname description="Number of Employees" languagecode="1033" /></displaynames></field><field name="address1_line2" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address 1: Street 2" languagecode="1033" /></displaynames></field><field name="timespentbymeonemailandmeetings" requiredlevel="none" format="text" maxlength="1250"><displaynames><displayname description="Time Spent by me" languagecode="1033" /></displaynames></field><field name="yominame" requiredlevel="none" imemode="active" format="phoneticguide" maxlength="160"><displaynames><displayname description="Yomi Account Name" languagecode="1033" /></displaynames></field><field name="address2_longitude" requiredlevel="none" imemode="disabled" minvalue="-180" maxvalue="180" accuracy="5"><displaynames><displayname description="Address 2: Longitude" languagecode="1033" /></displaynames></field><field name="ownershipcode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Public" languagecode="1033" /></labels></option><option value="2"><labels><label description="Private" languagecode="1033" /></labels></option><option value="3"><labels><label description="Subsidiary" languagecode="1033" /></labels></option><option value="4"><labels><label description="Other" languagecode="1033" /></labels></option></options><displaynames><displayname description="Ownership" languagecode="1033" /></displaynames></field><field name="openrevenue_base" requiredlevel="none" sourcetype="2" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Open Revenue (Base)" languagecode="1033" /></displaynames></field><field name="primarysatoriid" requiredlevel="none" format="text" maxlength="200"><displaynames><displayname description="Primary Satori ID" languagecode="1033" /></displaynames></field><field name="address1_telephone2" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 1: Telephone 2" languagecode="1033" /></displaynames></field><field name="address2_fax" requiredlevel="none" imemode="inactive" format="text" maxlength="50"><displaynames><displayname description="Address 2: Fax" languagecode="1033" /></displaynames></field><field name="slaid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="9750"><displaynames><displayname description="SLA" languagecode="1033" /></displaynames></field><field name="aging60_base" requiredlevel="none" imemode="disabled" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Aging 60 (Base)" languagecode="1033" /></displaynames></field><field name="telephone1" requiredlevel="none" issortableenabled="true" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Main Phone" languagecode="1033" /></displaynames></field><field name="accountcategorycode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Preferred Customer" languagecode="1033" /></labels></option><option value="2"><labels><label description="Standard" languagecode="1033" /></labels></option></options><displaynames><displayname description="Category" languagecode="1033" /></displaynames></field><field name="revenue" requiredlevel="none" imemode="disabled" minvalue="0" maxvalue="100000000000000" accuracy="2"><displaynames><displayname description="Annual Revenue" languagecode="1033" /></displaynames></field><field name="createdon" requiredlevel="none" isglobalfilterenabled="true" imemode="inactive" format="datetime" Behavior="Dynamic"><displaynames><displayname description="Created On" languagecode="1033" /></displaynames></field><field name="emailaddress1" requiredlevel="none" issortableenabled="true" imemode="inactive" format="email" maxlength="100"><displaynames><displayname description="Email" languagecode="1033" /></displaynames></field><field name="address1_latitude" requiredlevel="none" imemode="disabled" minvalue="-90" maxvalue="90" accuracy="5"><displaynames><displayname description="Address 1: Latitude" languagecode="1033" /></displaynames></field><field name="transactioncurrencyid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="9105"><displaynames><displayname description="Currency" languagecode="1033" /></displaynames></field><field name="address2_shippingmethodcode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 2: Shipping Method" languagecode="1033" /></displaynames></field><field name="opendeals" requiredlevel="none" issecured="true" sourcetype="2" format="" minvalue="-2147483648" maxvalue="2147483647"><displaynames><displayname description="Open Deals" languagecode="1033" /></displaynames></field><field name="address2_telephone1" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 2: Telephone 1" languagecode="1033" /></displaynames></field><field name="address2_name" requiredlevel="none" imemode="active" format="text" maxlength="200"><displaynames><displayname description="Address 2: Name" languagecode="1033" /></displaynames></field><field name="accountnumber" requiredlevel="none" format="text" maxlength="20"><displaynames><displayname description="Account Number" languagecode="1033" /></displaynames></field><field name="address1_stateorprovince" requiredlevel="none" imemode="active" format="text" maxlength="50"><displaynames><displayname description="Address 1: State/Province" languagecode="1033" /></displaynames></field><field name="address1_line3" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address 1: Street 3" languagecode="1033" /></displaynames></field><field name="industrycode" requiredlevel="none" isglobalfilterenabled="true" defaultvalue="-1"><options><option value="1"><labels><label description="Accounting" languagecode="1033" /></labels></option><option value="2"><labels><label description="Agriculture and Non-petrol Natural Resource Extraction" languagecode="1033" /></labels></option><option value="3"><labels><label description="Broadcasting Printing and Publishing" languagecode="1033" /></labels></option><option value="4"><labels><label description="Brokers" languagecode="1033" /></labels></option><option value="5"><labels><label description="Building Supply Retail" languagecode="1033" /></labels></option><option value="6"><labels><label description="Business Services" languagecode="1033" /></labels></option><option value="7"><labels><label description="Consulting" languagecode="1033" /></labels></option><option value="8"><labels><label description="Consumer Services" languagecode="1033" /></labels></option><option value="9"><labels><label description="Design, Direction and Creative Management" languagecode="1033" /></labels></option><option value="10"><labels><label description="Distributors, Dispatchers and Processors" languagecode="1033" /></labels></option><option value="11"><labels><label description="Doctors Offices and Clinics" languagecode="1033" /></labels></option><option value="12"><labels><label description="Durable Manufacturing" languagecode="1033" /></labels></option><option value="13"><labels><label description="Eating and Drinking Places" languagecode="1033" /></labels></option><option value="14"><labels><label description="Entertainment Retail" languagecode="1033" /></labels></option><option value="15"><labels><label description="Equipment Rental and Leasing" languagecode="1033" /></labels></option><option value="16"><labels><label description="Financial" languagecode="1033" /></labels></option><option value="17"><labels><label description="Food and Tobacco Processing" languagecode="1033" /></labels></option><option value="18"><labels><label description="Inbound Capital Intensive Processing" languagecode="1033" /></labels></option><option value="19"><labels><label description="Inbound Repair and Services" languagecode="1033" /></labels></option><option value="20"><labels><label description="Insurance" languagecode="1033" /></labels></option><option value="21"><labels><label description="Legal Services" languagecode="1033" /></labels></option><option value="22"><labels><label description="Non-Durable Merchandise Retail" languagecode="1033" /></labels></option><option value="23"><labels><label description="Outbound Consumer Service" languagecode="1033" /></labels></option><option value="24"><labels><label description="Petrochemical Extraction and Distribution" languagecode="1033" /></labels></option><option value="25"><labels><label description="Service Retail" languagecode="1033" /></labels></option><option value="26"><labels><label description="SIG Affiliations" languagecode="1033" /></labels></option><option value="27"><labels><label description="Social Services" languagecode="1033" /></labels></option><option value="28"><labels><label description="Special Outbound Trade Contractors" languagecode="1033" /></labels></option><option value="29"><labels><label description="Specialty Realty" languagecode="1033" /></labels></option><option value="30"><labels><label description="Transportation" languagecode="1033" /></labels></option><option value="31"><labels><label description="Utility Creation and Distribution" languagecode="1033" /></labels></option><option value="32"><labels><label description="Vehicle Retail" languagecode="1033" /></labels></option><option value="33"><labels><label description="Wholesale" languagecode="1033" /></labels></option></options><displaynames><displayname description="Industry" languagecode="1033" /></displaynames></field><field name="accountratingcode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Account Rating" languagecode="1033" /></displaynames></field><field name="address1_line1" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address 1: Street 1" languagecode="1033" /></displaynames></field><field name="importsequencenumber" requiredlevel="none" imemode="disabled" format="" minvalue="-2147483648" maxvalue="2147483647"><displaynames><displayname description="Import Sequence Number" languagecode="1033" /></displaynames></field><field name="paymenttermscode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Net 30" languagecode="1033" /></labels></option><option value="2"><labels><label description="2% 10, Net 30" languagecode="1033" /></labels></option><option value="3"><labels><label description="Net 45" languagecode="1033" /></labels></option><option value="4"><labels><label description="Net 60" languagecode="1033" /></labels></option></options><displaynames><displayname description="Payment Terms" languagecode="1033" /></displaynames></field><field name="fax" requiredlevel="none" imemode="inactive" format="text" maxlength="50"><displaynames><displayname description="Fax" languagecode="1033" /></displaynames></field><field name="businesstypecode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Business Type" languagecode="1033" /></displaynames></field><field name="address2_line3" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address 2: Street 3" languagecode="1033" /></displaynames></field><field name="address2_utcoffset" requiredlevel="none" imemode="disabled" format="timezone" minvalue="-1500" maxvalue="1500"><displaynames><displayname description="Address 2: UTC Offset" languagecode="1033" /></displaynames></field><field name="donotphone" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Allow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Do Not Allow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Do not allow Phone Calls" languagecode="1033" /></displaynames></field><field name="exchangerate" requiredlevel="none" imemode="disabled" minvalue="1E-10" maxvalue="100000000000" accuracy="10"><displaynames><displayname description="Exchange Rate" languagecode="1033" /></displaynames></field><field name="opendeals_state" requiredlevel="none" imemode="disabled" format="" minvalue="-2147483648" maxvalue="2147483647"><displaynames><displayname description="Open Deals (State)" languagecode="1033" /></displaynames></field><field name="modifiedonbehalfby" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Modified By (Delegate)" languagecode="1033" /></displaynames></field><field name="ftpsiteurl" requiredlevel="none" imemode="inactive" format="url" maxlength="200"><displaynames><displayname description="FTP Site" languagecode="1033" /></displaynames></field><field name="shippingmethodcode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Shipping Method" languagecode="1033" /></displaynames></field><field name="stockexchange" requiredlevel="none" format="text" maxlength="20"><displaynames><displayname description="Stock Exchange" languagecode="1033" /></displaynames></field><field name="address2_telephone3" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 2: Telephone 3" languagecode="1033" /></displaynames></field><field name="aging60" requiredlevel="none" imemode="disabled" minvalue="0" maxvalue="100000000000000" accuracy="2"><displaynames><displayname description="Aging 60" languagecode="1033" /></displaynames></field><field name="address1_postalcode" requiredlevel="none" isglobalfilterenabled="true" issortableenabled="true" imemode="inactive" format="text" maxlength="20"><displaynames><displayname description="Address 1: ZIP/Postal Code" languagecode="1033" /></displaynames></field><field name="donotpostalmail" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Allow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Do Not Allow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Do not allow Mails" languagecode="1033" /></displaynames></field><field name="address2_addresstypecode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 2: Address Type" languagecode="1033" /></displaynames></field><field name="aging90" requiredlevel="none" imemode="disabled" minvalue="0" maxvalue="100000000000000" accuracy="2"><displaynames><displayname description="Aging 90" languagecode="1033" /></displaynames></field><field name="donotbulkemail" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Allow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Do Not Allow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Do not allow Bulk Emails" languagecode="1033" /></displaynames></field><field name="aging30" requiredlevel="none" imemode="disabled" minvalue="0" maxvalue="100000000000000" accuracy="2"><displaynames><displayname description="Aging 30" languagecode="1033" /></displaynames></field><field name="sharesoutstanding" requiredlevel="none" imemode="disabled" format="" minvalue="0" maxvalue="1000000000"><displaynames><displayname description="Shares Outstanding" languagecode="1033" /></displaynames></field><field name="address2_latitude" requiredlevel="none" imemode="disabled" minvalue="-90" maxvalue="90" accuracy="5"><displaynames><displayname description="Address 2: Latitude" languagecode="1033" /></displaynames></field><field name="stageid" requiredlevel="none"><displaynames><displayname description="Process Stage" languagecode="1033" /></displaynames></field><field name="preferredappointmenttimecode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Morning" languagecode="1033" /></labels></option><option value="2"><labels><label description="Afternoon" languagecode="1033" /></labels></option><option value="3"><labels><label description="Evening" languagecode="1033" /></labels></option></options><displaynames><displayname description="Preferred Time" languagecode="1033" /></displaynames></field><field name="address2_composite" requiredlevel="none" imemode="active" format="textarea" maxlength="1000"><displaynames><displayname description="Address 2" languagecode="1033" /></displaynames></field><field name="aging90_base" requiredlevel="none" imemode="disabled" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Aging 90 (Base)" languagecode="1033" /></displaynames></field><field name="address1_fax" requiredlevel="none" imemode="inactive" format="text" maxlength="50"><displaynames><displayname description="Address 1: Fax" languagecode="1033" /></displaynames></field><field name="originatingleadid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="4"><displaynames><displayname description="Originating Lead" languagecode="1033" /></displaynames></field><field name="address1_city" requiredlevel="none" isglobalfilterenabled="true" issortableenabled="true" imemode="active" format="text" maxlength="80"><displaynames><displayname description="Address 1: City" languagecode="1033" /></displaynames></field><field name="description" requiredlevel="none" format="textarea" maxlength="2000"><displaynames><displayname description="Description" languagecode="1033" /></displaynames></field><field name="address2_stateorprovince" requiredlevel="none" imemode="active" format="text" maxlength="50"><displaynames><displayname description="Address 2: State/Province" languagecode="1033" /></displaynames></field><field name="donotfax" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Allow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Do Not Allow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Do not allow Faxes" languagecode="1033" /></displaynames></field><field name="overriddencreatedon" requiredlevel="none" imemode="inactive" format="date" Behavior="Dynamic"><displaynames><displayname description="Record Created On" languagecode="1033" /></displaynames></field><field name="ownerid" requiredlevel="systemrequired" isglobalfilterenabled="true" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8,9"><displaynames><displayname description="Owner" languagecode="1033" /></displaynames></field><field name="donotemail" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Allow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Do Not Allow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Do not allow Emails" languagecode="1033" /></displaynames></field><field name="address1_primarycontactname" requiredlevel="none" format="text" maxlength="100"><displaynames><displayname description="Address 1: Primary Contact Name" languagecode="1033" /></displaynames></field><field name="onholdtime" requiredlevel="none" imemode="disabled" format="" minvalue="-2147483648" maxvalue="2147483647"><displaynames><displayname description="On Hold Time (Minutes)" languagecode="1033" /></displaynames></field><field name="createdbyexternalparty" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="3008"><displaynames><displayname description="Created By (External Party)" languagecode="1033" /></displaynames></field><field name="marketingonly" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="No" languagecode="1033" /></labels></option><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option></options><displaynames><displayname description="Marketing Only" languagecode="1033" /></displaynames></field><field name="statuscode" requiredlevel="none" defaultvalue="-1"><statuses><status value="1" state="0"><labels><label description="Active" languagecode="1033" /></labels></status><status value="2" state="1"><labels><label description="Inactive" languagecode="1033" /></labels></status></statuses><displaynames><displayname description="Status Reason" languagecode="1033" /></displaynames></field><field name="processid" requiredlevel="none"><displaynames><displayname description="Process" languagecode="1033" /></displaynames></field><field name="createdonbehalfby" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Created By (Delegate)" languagecode="1033" /></displaynames></field><field name="modifiedbyexternalparty" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="3008"><displaynames><displayname description="Modified By (External Party)" languagecode="1033" /></displaynames></field><field name="address2_postalcode" requiredlevel="none" imemode="inactive" format="text" maxlength="20"><displaynames><displayname description="Address 2: ZIP/Postal Code" languagecode="1033" /></displaynames></field><field name="modifiedby" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Modified By" languagecode="1033" /></displaynames></field><field name="address1_shippingmethodcode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Airborne" languagecode="1033" /></labels></option><option value="2"><labels><label description="DHL" languagecode="1033" /></labels></option><option value="3"><labels><label description="FedEx" languagecode="1033" /></labels></option><option value="4"><labels><label description="UPS" languagecode="1033" /></labels></option><option value="5"><labels><label description="Postal Mail" languagecode="1033" /></labels></option><option value="6"><labels><label description="Full Load" languagecode="1033" /></labels></option><option value="7"><labels><label description="Will Call" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 1: Shipping Method" languagecode="1033" /></displaynames></field><field name="createdby" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Created By" languagecode="1033" /></displaynames></field><field name="openrevenue_date" requiredlevel="none" imemode="inactive" format="datetime" Behavior="Dynamic"><displaynames><displayname description="Open Revenue (Last Updated On)" languagecode="1033" /></displaynames></field><field name="address1_upszone" requiredlevel="none" format="text" maxlength="4"><displaynames><displayname description="Address 1: UPS Zone" languagecode="1033" /></displaynames></field><field name="address2_primarycontactname" requiredlevel="none" format="text" maxlength="100"><displaynames><displayname description="Address 2: Primary Contact Name" languagecode="1033" /></displaynames></field><field name="donotsendmm" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Send" languagecode="1033" /></labels></option><option value="1"><labels><label description="Do Not Send" languagecode="1033" /></labels></option></options><displaynames><displayname description="Send Marketing Materials" languagecode="1033" /></displaynames></field><field name="primarytwitterid" requiredlevel="none" format="text" maxlength="128"><displaynames><displayname description="Primary Twitter ID" languagecode="1033" /></displaynames></field><field name="preferredserviceid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="4001"><displaynames><displayname description="Preferred Service" languagecode="1033" /></displaynames></field><field name="customertypecode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Competitor" languagecode="1033" /></labels></option><option value="2"><labels><label description="Consultant" languagecode="1033" /></labels></option><option value="3"><labels><label description="Customer" languagecode="1033" /></labels></option><option value="4"><labels><label description="Investor" languagecode="1033" /></labels></option><option value="5"><labels><label description="Partner" languagecode="1033" /></labels></option><option value="6"><labels><label description="Influencer" languagecode="1033" /></labels></option><option value="7"><labels><label description="Press" languagecode="1033" /></labels></option><option value="8"><labels><label description="Prospect" languagecode="1033" /></labels></option><option value="9"><labels><label description="Reseller" languagecode="1033" /></labels></option><option value="10"><labels><label description="Supplier" languagecode="1033" /></labels></option><option value="11"><labels><label description="Vendor" languagecode="1033" /></labels></option><option value="12"><labels><label description="Other" languagecode="1033" /></labels></option></options><displaynames><displayname description="Relationship Type" languagecode="1033" /></displaynames></field><field name="address2_postofficebox" requiredlevel="none" imemode="inactive" format="text" maxlength="20"><displaynames><displayname description="Address 2: Post Office Box" languagecode="1033" /></displaynames></field><field name="statecode" requiredlevel="systemrequired" defaultvalue="0"><states><state value="0" defaultstatus="1" invariantname="Active"><labels><label description="Active" languagecode="1033" /></labels></state><state value="1" defaultstatus="2" invariantname="Inactive"><labels><label description="Inactive" languagecode="1033" /></labels></state></states><displaynames><displayname description="Status" languagecode="1033" /></displaynames></field><field name="address2_city" requiredlevel="none" imemode="active" format="text" maxlength="80"><displaynames><displayname description="Address 2: City" languagecode="1033" /></displaynames></field><field name="primarycontactid" requiredlevel="none" isglobalfilterenabled="true" issortableenabled="true" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="2"><displaynames><displayname description="Primary Contact" languagecode="1033" /></displaynames></field><field name="address1_freighttermscode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="FOB" languagecode="1033" /></labels></option><option value="2"><labels><label description="No Charge" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 1: Freight Terms" languagecode="1033" /></displaynames></field><field name="openrevenue_state" requiredlevel="none" imemode="disabled" format="" minvalue="-2147483648" maxvalue="2147483647"><displaynames><displayname description="Open Revenue (State)" languagecode="1033" /></displaynames></field><field name="address1_longitude" requiredlevel="none" imemode="disabled" minvalue="-180" maxvalue="180" accuracy="5"><displaynames><displayname description="Address 1: Longitude" languagecode="1033" /></displaynames></field><field name="customersizecode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Customer Size" languagecode="1033" /></displaynames></field><field name="telephone2" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Other Phone" languagecode="1033" /></displaynames></field><field name="address1_addresstypecode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Bill To" languagecode="1033" /></labels></option><option value="2"><labels><label description="Ship To" languagecode="1033" /></labels></option><option value="3"><labels><label description="Primary" languagecode="1033" /></labels></option><option value="4"><labels><label description="Other" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 1: Address Type" languagecode="1033" /></displaynames></field><field name="donotbulkpostalmail" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="No" languagecode="1033" /></labels></option><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option></options><displaynames><displayname description="Do not allow Bulk Mails" languagecode="1033" /></displaynames></field><field name="lastusedincampaign" requiredlevel="none" imemode="inactive" format="date" Behavior="Dynamic"><displaynames><displayname description="Last Date Included in Campaign" languagecode="1033" /></displaynames></field><field name="preferredsystemuserid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Preferred User" languagecode="1033" /></displaynames></field><field name="creditlimit_base" requiredlevel="none" imemode="disabled" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Credit Limit (Base)" languagecode="1033" /></displaynames></field><field name="telephone3" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Telephone 3" languagecode="1033" /></displaynames></field><field name="preferredappointmentdaycode" requiredlevel="none" defaultvalue="-1"><options><option value="0"><labels><label description="Sunday" languagecode="1033" /></labels></option><option value="1"><labels><label description="Monday" languagecode="1033" /></labels></option><option value="2"><labels><label description="Tuesday" languagecode="1033" /></labels></option><option value="3"><labels><label description="Wednesday" languagecode="1033" /></labels></option><option value="4"><labels><label description="Thursday" languagecode="1033" /></labels></option><option value="5"><labels><label description="Friday" languagecode="1033" /></labels></option><option value="6"><labels><label description="Saturday" languagecode="1033" /></labels></option></options><displaynames><displayname description="Preferred Day" languagecode="1033" /></displaynames></field><field name="websiteurl" requiredlevel="none" imemode="inactive" format="url" maxlength="200"><displaynames><displayname description="Website" languagecode="1033" /></displaynames></field><field name="accountclassificationcode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Classification" languagecode="1033" /></displaynames></field><field name="defaultpricelevelid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="1022"><displaynames><displayname description="Price List" languagecode="1033" /></displaynames></field><field name="name" requiredlevel="required" issortableenabled="true" imemode="active" format="text" maxlength="160"><displaynames><displayname description="Account Name" languagecode="1033" /></displaynames></field><field name="sic" requiredlevel="none" format="text" maxlength="20"><displaynames><displayname description="SIC Code" languagecode="1033" /></displaynames></field><field name="preferredequipmentid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="4000"><displaynames><displayname description="Preferred Facility/Equipment" languagecode="1033" /></displaynames></field><field name="creditlimit" requiredlevel="none" imemode="disabled" minvalue="0" maxvalue="100000000000000" accuracy="2"><displaynames><displayname description="Credit Limit" languagecode="1033" /></displaynames></field><field name="lastonholdtime" requiredlevel="none" imemode="inactive" format="datetime" Behavior="Dynamic"><displaynames><displayname description="Last On Hold Time" languagecode="1033" /></displaynames></field><field name="address2_line2" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address 2: Street 2" languagecode="1033" /></displaynames></field><field name="tickersymbol" requiredlevel="none" format="tickersymbol" maxlength="10"><displaynames><displayname description="Ticker Symbol" languagecode="1033" /></displaynames></field><field name="creditonhold" requiredlevel="none" defaultvalue="0"><options><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option><option value="0"><labels><label description="No" languagecode="1033" /></labels></option></options><displaynames><displayname description="Credit Hold" languagecode="1033" /></displaynames></field><field name="aging30_base" requiredlevel="none" imemode="disabled" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Aging 30 (Base)" languagecode="1033" /></displaynames></field><field name="address1_composite" requiredlevel="none" imemode="active" format="textarea" maxlength="1000"><displaynames><displayname description="Address 1" languagecode="1033" /></displaynames></field><field name="territoryid" requiredlevel="none" isglobalfilterenabled="true" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="2013"><displaynames><displayname description="Territory" languagecode="1033" /></displaynames></field><field name="followemail" requiredlevel="none" defaultvalue="1"><options><option value="0"><labels><label description="Do Not Allow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Allow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Follow Email Activity" languagecode="1033" /></displaynames></field><field name="modifiedon" requiredlevel="none" imemode="inactive" format="datetime" Behavior="Dynamic"><displaynames><displayname description="Modified On" languagecode="1033" /></displaynames></field><field name="address1_county" requiredlevel="none" imemode="active" format="text" maxlength="50"><displaynames><displayname description="Address 1: County" languagecode="1033" /></displaynames></field><field name="marketcap_base" requiredlevel="none" imemode="disabled" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Market Capitalization (Base)" languagecode="1033" /></displaynames></field><field name="address2_upszone" requiredlevel="none" format="text" maxlength="4"><displaynames><displayname description="Address 2: UPS Zone" languagecode="1033" /></displaynames></field><field name="address2_country" requiredlevel="none" imemode="active" format="text" maxlength="80"><displaynames><displayname description="Address 2: Country/Region" languagecode="1033" /></displaynames></field><field name="address2_freighttermscode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 2: Freight Terms" languagecode="1033" /></displaynames></field><field name="address1_utcoffset" requiredlevel="none" imemode="disabled" format="timezone" minvalue="-1500" maxvalue="1500"><displaynames><displayname description="Address 1: UTC Offset" languagecode="1033" /></displaynames></field><field name="emailaddress2" requiredlevel="none" imemode="inactive" format="email" maxlength="100"><displaynames><displayname description="Email Address 2" languagecode="1033" /></displaynames></field><field name="parentaccountid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="1"><displaynames><displayname description="Parent Account" languagecode="1033" /></displaynames></field><field name="preferredcontactmethodcode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Any" languagecode="1033" /></labels></option><option value="2"><labels><label description="Email" languagecode="1033" /></labels></option><option value="3"><labels><label description="Phone" languagecode="1033" /></labels></option><option value="4"><labels><label description="Fax" languagecode="1033" /></labels></option><option value="5"><labels><label description="Mail" languagecode="1033" /></labels></option></options><displaynames><displayname description="Preferred Method of Contact" languagecode="1033" /></displaynames></field><field name="revenue_base" requiredlevel="none" imemode="disabled" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Annual Revenue (Base)" languagecode="1033" /></displaynames></field><field name="address2_telephone2" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 2: Telephone 2" languagecode="1033" /></displaynames></field><field name="address1_country" requiredlevel="none" imemode="active" format="text" maxlength="80"><displaynames><displayname description="Address 1: Country/Region" languagecode="1033" /></displaynames></field></fields></entity><entity name="contact" objecttypecode="2"><fields><field name="spousesname" requiredlevel="none" format="text" maxlength="100"><displaynames><displayname description="Spouse/Partner Name" languagecode="1033" /></displaynames></field><field name="emailaddress3" requiredlevel="none" imemode="inactive" format="email" maxlength="100"><displaynames><displayname description="Email Address 3" languagecode="1033" /></displaynames></field><field name="address3_telephone3" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 3: Telephone3" languagecode="1033" /></displaynames></field><field name="address2_shippingmethodcode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 2: Shipping Method" languagecode="1033" /></displaynames></field><field name="mobilephone" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Mobile Phone" languagecode="1033" /></displaynames></field><field name="preferredserviceid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="4001"><displaynames><displayname description="Preferred Service" languagecode="1033" /></displaynames></field><field name="address3_shippingmethodcode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 3: Shipping Method" languagecode="1033" /></displaynames></field><field name="address3_postalcode" requiredlevel="none" imemode="inactive" format="text" maxlength="20"><displaynames><displayname description="Address3: ZIP/Postal Code" languagecode="1033" /></displaynames></field><field name="annualincome" requiredlevel="none" imemode="disabled" minvalue="0" maxvalue="100000000000000" accuracy="2"><displaynames><displayname description="Annual Income" languagecode="1033" /></displaynames></field><field name="fax" requiredlevel="none" imemode="inactive" format="text" maxlength="50"><displaynames><displayname description="Fax" languagecode="1033" /></displaynames></field><field name="telephone3" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Telephone 3" languagecode="1033" /></displaynames></field><field name="address1_primarycontactname" requiredlevel="none" format="text" maxlength="100"><displaynames><displayname description="Address 1: Primary Contact Name" languagecode="1033" /></displaynames></field><field name="address3_city" requiredlevel="none" imemode="active" format="text" maxlength="80"><displaynames><displayname description="Address 3: City" languagecode="1033" /></displaynames></field><field name="lastonholdtime" requiredlevel="none" imemode="inactive" format="datetime" Behavior="Dynamic"><displaynames><displayname description="Last On Hold Time" languagecode="1033" /></displaynames></field><field name="address2_stateorprovince" requiredlevel="none" imemode="active" format="text" maxlength="50"><displaynames><displayname description="Address 2: State/Province" languagecode="1033" /></displaynames></field><field name="createdby" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Created By" languagecode="1033" /></displaynames></field><field name="address2_line1" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address 2: Street 1" languagecode="1033" /></displaynames></field><field name="assistantphone" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Assistant Phone" languagecode="1033" /></displaynames></field><field name="lastusedincampaign" requiredlevel="none" imemode="inactive" format="date" Behavior="Dynamic"><displaynames><displayname description="Last Date Included in Campaign" languagecode="1033" /></displaynames></field><field name="address3_freighttermscode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 3: Freight Terms" languagecode="1033" /></displaynames></field><field name="pager" requiredlevel="none" imemode="inactive" format="text" maxlength="50"><displaynames><displayname description="Pager" languagecode="1033" /></displaynames></field><field name="employeeid" requiredlevel="none" format="text" maxlength="50"><displaynames><displayname description="Employee" languagecode="1033" /></displaynames></field><field name="address1_latitude" requiredlevel="none" imemode="disabled" minvalue="-90" maxvalue="90" accuracy="5"><displaynames><displayname description="Address 1: Latitude" languagecode="1033" /></displaynames></field><field name="parentcustomerid" requiredlevel="none" isglobalfilterenabled="true" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="1,2"><displaynames><displayname description="Company Name" languagecode="1033" /></displaynames></field><field name="managername" requiredlevel="none" format="text" maxlength="100"><displaynames><displayname description="Manager" languagecode="1033" /></displaynames></field><field name="birthdate" requiredlevel="none" imemode="inactive" format="date" Behavior="DateOnly"><displaynames><displayname description="Birthday" languagecode="1033" /></displaynames></field><field name="address1_name" requiredlevel="none" imemode="active" format="text" maxlength="200"><displaynames><displayname description="Address 1: Name" languagecode="1033" /></displaynames></field><field name="department" requiredlevel="none" format="text" maxlength="100"><displaynames><displayname description="Department" languagecode="1033" /></displaynames></field><field name="address2_telephone1" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 2: Telephone 1" languagecode="1033" /></displaynames></field><field name="address2_primarycontactname" requiredlevel="none" format="text" maxlength="100"><displaynames><displayname description="Address 2: Primary Contact Name" languagecode="1033" /></displaynames></field><field name="address2_latitude" requiredlevel="none" imemode="disabled" minvalue="-90" maxvalue="90" accuracy="5"><displaynames><displayname description="Address 2: Latitude" languagecode="1033" /></displaynames></field><field name="territorycode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Territory" languagecode="1033" /></displaynames></field><field name="address2_postalcode" requiredlevel="none" imemode="inactive" format="text" maxlength="20"><displaynames><displayname description="Address 2: ZIP/Postal Code" languagecode="1033" /></displaynames></field><field name="originatingleadid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="4"><displaynames><displayname description="Originating Lead" languagecode="1033" /></displaynames></field><field name="createdonbehalfby" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Created By (Delegate)" languagecode="1033" /></displaynames></field><field name="business2" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Business Phone 2" languagecode="1033" /></displaynames></field><field name="address3_county" requiredlevel="none" imemode="active" format="text" maxlength="50"><displaynames><displayname description="Address 3: County" languagecode="1033" /></displaynames></field><field name="address1_freighttermscode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="FOB" languagecode="1033" /></labels></option><option value="2"><labels><label description="No Charge" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 1: Freight Terms" languagecode="1033" /></displaynames></field><field name="customertypecode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Relationship Type" languagecode="1033" /></displaynames></field><field name="address3_addresstypecode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 3: Address Type" languagecode="1033" /></displaynames></field><field name="address1_longitude" requiredlevel="none" imemode="disabled" minvalue="-180" maxvalue="180" accuracy="5"><displaynames><displayname description="Address 1: Longitude" languagecode="1033" /></displaynames></field><field name="address1_addresstypecode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Bill To" languagecode="1033" /></labels></option><option value="2"><labels><label description="Ship To" languagecode="1033" /></labels></option><option value="3"><labels><label description="Primary" languagecode="1033" /></labels></option><option value="4"><labels><label description="Other" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 1: Address Type" languagecode="1033" /></displaynames></field><field name="statuscode" requiredlevel="none" defaultvalue="-1"><statuses><status value="1" state="0"><labels><label description="Active" languagecode="1033" /></labels></status><status value="2" state="1"><labels><label description="Inactive" languagecode="1033" /></labels></status></statuses><displaynames><displayname description="Status Reason" languagecode="1033" /></displaynames></field><field name="yomifullname" requiredlevel="none" imemode="active" format="phoneticguide" maxlength="450"><displaynames><displayname description="Yomi Full Name" languagecode="1033" /></displaynames></field><field name="aging90_base" requiredlevel="none" imemode="disabled" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Aging 90 (Base)" languagecode="1033" /></displaynames></field><field name="address3_primarycontactname" requiredlevel="none" format="text" maxlength="100"><displaynames><displayname description="Address 3: Primary Contact Name" languagecode="1033" /></displaynames></field><field name="familystatuscode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Single" languagecode="1033" /></labels></option><option value="2"><labels><label description="Married" languagecode="1033" /></labels></option><option value="3"><labels><label description="Divorced" languagecode="1033" /></labels></option><option value="4"><labels><label description="Widowed" languagecode="1033" /></labels></option></options><displaynames><displayname description="Marital Status" languagecode="1033" /></displaynames></field><field name="home2" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Home Phone 2" languagecode="1033" /></displaynames></field><field name="address2_utcoffset" requiredlevel="none" imemode="disabled" format="timezone" minvalue="-1500" maxvalue="1500"><displaynames><displayname description="Address 2: UTC Offset" languagecode="1033" /></displaynames></field><field name="aging60" requiredlevel="none" imemode="disabled" minvalue="0" maxvalue="100000000000000" accuracy="2"><displaynames><displayname description="Aging 60" languagecode="1033" /></displaynames></field><field name="statecode" requiredlevel="systemrequired" defaultvalue="0"><states><state value="0" defaultstatus="1" invariantname="Active"><labels><label description="Active" languagecode="1033" /></labels></state><state value="1" defaultstatus="2" invariantname="Inactive"><labels><label description="Inactive" languagecode="1033" /></labels></state></states><displaynames><displayname description="Status" languagecode="1033" /></displaynames></field><field name="address2_freighttermscode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 2: Freight Terms" languagecode="1033" /></displaynames></field><field name="address1_composite" requiredlevel="none" imemode="active" format="textarea" maxlength="1000"><displaynames><displayname description="Address 1" languagecode="1033" /></displaynames></field><field name="telephone2" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Home Phone" languagecode="1033" /></displaynames></field><field name="yomimiddlename" requiredlevel="none" imemode="active" format="phoneticguide" maxlength="150"><displaynames><displayname description="Yomi Middle Name" languagecode="1033" /></displaynames></field><field name="modifiedonbehalfby" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Modified By (Delegate)" languagecode="1033" /></displaynames></field><field name="jobtitle" requiredlevel="none" format="text" maxlength="100"><displaynames><displayname description="Job Title" languagecode="1033" /></displaynames></field><field name="address3_utcoffset" requiredlevel="none" imemode="disabled" format="timezone" minvalue="-1500" maxvalue="1500"><displaynames><displayname description="Address 3: UTC Offset" languagecode="1033" /></displaynames></field><field name="address2_addresstypecode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 2: Address Type" languagecode="1033" /></displaynames></field><field name="educationcode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Education" languagecode="1033" /></displaynames></field><field name="address1_telephone3" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 1: Telephone 3" languagecode="1033" /></displaynames></field><field name="followemail" requiredlevel="none" defaultvalue="1"><options><option value="0"><labels><label description="Do Not Allow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Allow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Follow Email Activity" languagecode="1033" /></displaynames></field><field name="preferredcontactmethodcode" requiredlevel="none" isglobalfilterenabled="true" defaultvalue="1"><options><option value="1"><labels><label description="Any" languagecode="1033" /></labels></option><option value="2"><labels><label description="Email" languagecode="1033" /></labels></option><option value="3"><labels><label description="Phone" languagecode="1033" /></labels></option><option value="4"><labels><label description="Fax" languagecode="1033" /></labels></option><option value="5"><labels><label description="Mail" languagecode="1033" /></labels></option></options><displaynames><displayname description="Preferred Method of Contact" languagecode="1033" /></displaynames></field><field name="importsequencenumber" requiredlevel="none" imemode="disabled" format="" minvalue="-2147483648" maxvalue="2147483647"><displaynames><displayname description="Import Sequence Number" languagecode="1033" /></displaynames></field><field name="gendercode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Male" languagecode="1033" /></labels></option><option value="2"><labels><label description="Female" languagecode="1033" /></labels></option></options><displaynames><displayname description="Gender" languagecode="1033" /></displaynames></field><field name="address2_line2" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address 2: Street 2" languagecode="1033" /></displaynames></field><field name="creditlimit_base" requiredlevel="none" imemode="disabled" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Credit Limit (Base)" languagecode="1033" /></displaynames></field><field name="donotfax" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Allow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Do Not Allow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Do not allow Faxes" languagecode="1033" /></displaynames></field><field name="address3_line1" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address3: Street 1" languagecode="1033" /></displaynames></field><field name="address1_county" requiredlevel="none" imemode="active" format="text" maxlength="50"><displaynames><displayname description="Address 1: County" languagecode="1033" /></displaynames></field><field name="createdbyexternalparty" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="3008"><displaynames><displayname description="Created By (External Party)" languagecode="1033" /></displaynames></field><field name="processid" requiredlevel="none"><displaynames><displayname description="Process" languagecode="1033" /></displaynames></field><field name="address1_telephone2" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 1: Telephone 2" languagecode="1033" /></displaynames></field><field name="description" requiredlevel="none" format="textarea" maxlength="2000"><displaynames><displayname description="Description" languagecode="1033" /></displaynames></field><field name="address1_fax" requiredlevel="none" imemode="inactive" format="text" maxlength="50"><displaynames><displayname description="Address 1: Fax" languagecode="1033" /></displaynames></field><field name="marketingonly" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="No" languagecode="1033" /></labels></option><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option></options><displaynames><displayname description="Marketing Only" languagecode="1033" /></displaynames></field><field name="address3_line2" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address3: Street 2" languagecode="1033" /></displaynames></field><field name="donotpostalmail" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Allow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Do Not Allow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Do not allow Mails" languagecode="1033" /></displaynames></field><field name="aging30_base" requiredlevel="none" imemode="disabled" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Aging 30 (Base)" languagecode="1033" /></displaynames></field><field name="callback" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Callback Number" languagecode="1033" /></displaynames></field><field name="emailaddress2" requiredlevel="none" imemode="inactive" format="email" maxlength="100"><displaynames><displayname description="Email Address 2" languagecode="1033" /></displaynames></field><field name="address2_line3" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address 2: Street 3" languagecode="1033" /></displaynames></field><field name="managerphone" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Manager Phone" languagecode="1033" /></displaynames></field><field name="preferredappointmentdaycode" requiredlevel="none" defaultvalue="-1"><options><option value="0"><labels><label description="Sunday" languagecode="1033" /></labels></option><option value="1"><labels><label description="Monday" languagecode="1033" /></labels></option><option value="2"><labels><label description="Tuesday" languagecode="1033" /></labels></option><option value="3"><labels><label description="Wednesday" languagecode="1033" /></labels></option><option value="4"><labels><label description="Thursday" languagecode="1033" /></labels></option><option value="5"><labels><label description="Friday" languagecode="1033" /></labels></option><option value="6"><labels><label description="Saturday" languagecode="1033" /></labels></option></options><displaynames><displayname description="Preferred Day" languagecode="1033" /></displaynames></field><field name="websiteurl" requiredlevel="none" imemode="inactive" format="url" maxlength="200"><displaynames><displayname description="Website" languagecode="1033" /></displaynames></field><field name="exchangerate" requiredlevel="none" imemode="disabled" minvalue="1E-10" maxvalue="100000000000" accuracy="10"><displaynames><displayname description="Exchange Rate" languagecode="1033" /></displaynames></field><field name="firstname" requiredlevel="recommended" imemode="active" format="text" maxlength="50"><displaynames><displayname description="First Name" languagecode="1033" /></displaynames></field><field name="address1_telephone1" requiredlevel="none" issortableenabled="true" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 1: Phone" languagecode="1033" /></displaynames></field><field name="address3_composite" requiredlevel="none" imemode="active" format="textarea" maxlength="1000"><displaynames><displayname description="Address 3" languagecode="1033" /></displaynames></field><field name="address3_fax" requiredlevel="none" imemode="inactive" format="text" maxlength="50"><displaynames><displayname description="Address 3: Fax" languagecode="1033" /></displaynames></field><field name="childrensnames" requiredlevel="none" format="text" maxlength="255"><displaynames><displayname description="Childrens Names" languagecode="1033" /></displaynames></field><field name="numberofchildren" requiredlevel="none" imemode="disabled" format="" minvalue="0" maxvalue="1000000000"><displaynames><displayname description="No. of Children" languagecode="1033" /></displaynames></field><field name="address2_postofficebox" requiredlevel="none" imemode="inactive" format="text" maxlength="20"><displaynames><displayname description="Address 2: Post Office Box" languagecode="1033" /></displaynames></field><field name="aging90" requiredlevel="none" imemode="disabled" minvalue="0" maxvalue="100000000000000" accuracy="2"><displaynames><displayname description="Aging 90" languagecode="1033" /></displaynames></field><field name="donotbulkpostalmail" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="No" languagecode="1033" /></labels></option><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option></options><displaynames><displayname description="Do not allow Bulk Mails" languagecode="1033" /></displaynames></field><field name="emailaddress1" requiredlevel="none" issortableenabled="true" imemode="inactive" format="email" maxlength="100"><displaynames><displayname description="Email" languagecode="1033" /></displaynames></field><field name="donotbulkemail" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Allow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Do Not Allow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Do not allow Bulk Emails" languagecode="1033" /></displaynames></field><field name="customersizecode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Customer Size" languagecode="1033" /></displaynames></field><field name="address1_city" requiredlevel="none" isglobalfilterenabled="true" issortableenabled="true" imemode="active" format="text" maxlength="80"><displaynames><displayname description="Address 1: City" languagecode="1033" /></displaynames></field><field name="preferredappointmenttimecode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Morning" languagecode="1033" /></labels></option><option value="2"><labels><label description="Afternoon" languagecode="1033" /></labels></option><option value="3"><labels><label description="Evening" languagecode="1033" /></labels></option></options><displaynames><displayname description="Preferred Time" languagecode="1033" /></displaynames></field><field name="address3_latitude" requiredlevel="none" imemode="disabled" minvalue="-90" maxvalue="90" accuracy="5"><displaynames><displayname description="Address 3: Latitude" languagecode="1033" /></displaynames></field><field name="aging60_base" requiredlevel="none" imemode="disabled" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Aging 60 (Base)" languagecode="1033" /></displaynames></field><field name="transactioncurrencyid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="9105"><displaynames><displayname description="Currency" languagecode="1033" /></displaynames></field><field name="modifiedbyexternalparty" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="3008"><displaynames><displayname description="Modified By (External Party)" languagecode="1033" /></displaynames></field><field name="paymenttermscode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Net 30" languagecode="1033" /></labels></option><option value="2"><labels><label description="2% 10, Net 30" languagecode="1033" /></labels></option><option value="3"><labels><label description="Net 45" languagecode="1033" /></labels></option><option value="4"><labels><label description="Net 60" languagecode="1033" /></labels></option></options><displaynames><displayname description="Payment Terms" languagecode="1033" /></displaynames></field><field name="address3_name" requiredlevel="none" imemode="active" format="text" maxlength="200"><displaynames><displayname description="Address 3: Name" languagecode="1033" /></displaynames></field><field name="leadsourcecode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Lead Source" languagecode="1033" /></displaynames></field><field name="ftpsiteurl" requiredlevel="none" imemode="inactive" format="url" maxlength="200"><displaynames><displayname description="FTP Site" languagecode="1033" /></displaynames></field><field name="address1_shippingmethodcode" requiredlevel="none" defaultvalue="-1"><options><option value="1"><labels><label description="Airborne" languagecode="1033" /></labels></option><option value="2"><labels><label description="DHL" languagecode="1033" /></labels></option><option value="3"><labels><label description="FedEx" languagecode="1033" /></labels></option><option value="4"><labels><label description="UPS" languagecode="1033" /></labels></option><option value="5"><labels><label description="Postal Mail" languagecode="1033" /></labels></option><option value="6"><labels><label description="Full Load" languagecode="1033" /></labels></option><option value="7"><labels><label description="Will Call" languagecode="1033" /></labels></option></options><displaynames><displayname description="Address 1: Shipping Method" languagecode="1033" /></displaynames></field><field name="modifiedby" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Modified By" languagecode="1033" /></displaynames></field><field name="preferredsystemuserid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Preferred User" languagecode="1033" /></displaynames></field><field name="address2_telephone2" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 2: Telephone 2" languagecode="1033" /></displaynames></field><field name="address3_telephone1" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 3: Telephone1" languagecode="1033" /></displaynames></field><field name="nickname" requiredlevel="none" format="text" maxlength="100"><displaynames><displayname description="Nickname" languagecode="1033" /></displaynames></field><field name="address1_postofficebox" requiredlevel="none" imemode="inactive" format="text" maxlength="20"><displaynames><displayname description="Address 1: Post Office Box" languagecode="1033" /></displaynames></field><field name="preferredequipmentid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="4000"><displaynames><displayname description="Preferred Facility/Equipment" languagecode="1033" /></displaynames></field><field name="assistantname" requiredlevel="none" format="text" maxlength="100"><displaynames><displayname description="Assistant" languagecode="1033" /></displaynames></field><field name="address2_country" requiredlevel="none" imemode="active" format="text" maxlength="80"><displaynames><displayname description="Address 2: Country/Region" languagecode="1033" /></displaynames></field><field name="modifiedon" requiredlevel="none" imemode="inactive" format="datetime" Behavior="Dynamic"><displaynames><displayname description="Modified On" languagecode="1033" /></displaynames></field><field name="stageid" requiredlevel="none"><displaynames><displayname description="Process Stage" languagecode="1033" /></displaynames></field><field name="creditonhold" requiredlevel="none" defaultvalue="0"><options><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option><option value="0"><labels><label description="No" languagecode="1033" /></labels></option></options><displaynames><displayname description="Credit Hold" languagecode="1033" /></displaynames></field><field name="address3_longitude" requiredlevel="none" imemode="disabled" minvalue="-180" maxvalue="180" accuracy="5"><displaynames><displayname description="Address 3: Longitude" languagecode="1033" /></displaynames></field><field name="onholdtime" requiredlevel="none" imemode="disabled" format="" minvalue="-2147483648" maxvalue="2147483647"><displaynames><displayname description="On Hold Time (Minutes)" languagecode="1033" /></displaynames></field><field name="address2_telephone3" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 2: Telephone 3" languagecode="1033" /></displaynames></field><field name="donotphone" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Allow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Do Not Allow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Do not allow Phone Calls" languagecode="1033" /></displaynames></field><field name="address3_upszone" requiredlevel="none" format="text" maxlength="4"><displaynames><displayname description="Address 3: UPS Zone" languagecode="1033" /></displaynames></field><field name="contactid" requiredlevel="systemrequired"><displaynames><displayname description="Contact" languagecode="1033" /></displaynames></field><field name="aging30" requiredlevel="none" imemode="disabled" minvalue="0" maxvalue="100000000000000" accuracy="2"><displaynames><displayname description="Aging 30" languagecode="1033" /></displaynames></field><field name="address2_upszone" requiredlevel="none" format="text" maxlength="4"><displaynames><displayname description="Address 2: UPS Zone" languagecode="1033" /></displaynames></field><field name="address1_upszone" requiredlevel="none" format="text" maxlength="4"><displaynames><displayname description="Address 1: UPS Zone" languagecode="1033" /></displaynames></field><field name="creditlimit" requiredlevel="none" imemode="disabled" minvalue="0" maxvalue="100000000000000" accuracy="2"><displaynames><displayname description="Credit Limit" languagecode="1033" /></displaynames></field><field name="accountrolecode" requiredlevel="none" isglobalfilterenabled="true" defaultvalue="-1"><options><option value="1"><labels><label description="Decision Maker" languagecode="1033" /></labels></option><option value="2"><labels><label description="Employee" languagecode="1033" /></labels></option><option value="3"><labels><label description="Influencer" languagecode="1033" /></labels></option></options><displaynames><displayname description="Role" languagecode="1033" /></displaynames></field><field name="salutation" requiredlevel="none" format="text" maxlength="100"><displaynames><displayname description="Salutation" languagecode="1033" /></displaynames></field><field name="suffix" requiredlevel="none" format="text" maxlength="10"><displaynames><displayname description="Suffix" languagecode="1033" /></displaynames></field><field name="address1_utcoffset" requiredlevel="none" imemode="disabled" format="timezone" minvalue="-1500" maxvalue="1500"><displaynames><displayname description="Address 1: UTC Offset" languagecode="1033" /></displaynames></field><field name="governmentid" requiredlevel="none" format="text" maxlength="50"><displaynames><displayname description="Government" languagecode="1033" /></displaynames></field><field name="fullname" requiredlevel="none" issortableenabled="true" imemode="active" format="text" maxlength="160"><displaynames><displayname description="Full Name" languagecode="1033" /></displaynames></field><field name="address1_stateorprovince" requiredlevel="none" imemode="active" format="text" maxlength="50"><displaynames><displayname description="Address 1: State/Province" languagecode="1033" /></displaynames></field><field name="donotsendmm" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Send" languagecode="1033" /></labels></option><option value="1"><labels><label description="Do Not Send" languagecode="1033" /></labels></option></options><displaynames><displayname description="Send Marketing Materials" languagecode="1033" /></displaynames></field><field name="annualincome_base" requiredlevel="none" imemode="disabled" minvalue="-922337203685477" maxvalue="922337203685477" accuracy="2"><displaynames><displayname description="Annual Income (Base)" languagecode="1033" /></displaynames></field><field name="address1_country" requiredlevel="none" imemode="active" format="text" maxlength="80"><displaynames><displayname description="Address 1: Country/Region" languagecode="1033" /></displaynames></field><field name="address3_stateorprovince" requiredlevel="none" imemode="active" format="text" maxlength="50"><displaynames><displayname description="Address3: State/Province" languagecode="1033" /></displaynames></field><field name="lastname" requiredlevel="required" imemode="active" format="text" maxlength="50"><displaynames><displayname description="Last Name" languagecode="1033" /></displaynames></field><field name="address2_city" requiredlevel="none" imemode="active" format="text" maxlength="80"><displaynames><displayname description="Address 2: City" languagecode="1033" /></displaynames></field><field name="donotemail" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Allow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Do Not Allow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Do not allow Emails" languagecode="1033" /></displaynames></field><field name="company" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Company Phone" languagecode="1033" /></displaynames></field><field name="address3_postofficebox" requiredlevel="none" imemode="inactive" format="text" maxlength="20"><displaynames><displayname description="Address 3: Post Office Box" languagecode="1033" /></displaynames></field><field name="address2_name" requiredlevel="none" imemode="active" format="text" maxlength="100"><displaynames><displayname description="Address 2: Name" languagecode="1033" /></displaynames></field><field name="address1_line2" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address 1: Street 2" languagecode="1033" /></displaynames></field><field name="address2_longitude" requiredlevel="none" imemode="disabled" minvalue="-180" maxvalue="180" accuracy="5"><displaynames><displayname description="Address 2: Longitude" languagecode="1033" /></displaynames></field><field name="address3_telephone2" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Address 3: Telephone2" languagecode="1033" /></displaynames></field><field name="yomifirstname" requiredlevel="none" imemode="active" format="phoneticguide" maxlength="150"><displaynames><displayname description="Yomi First Name" languagecode="1033" /></displaynames></field><field name="telephone1" requiredlevel="none" imemode="inactive" format="phone" maxlength="50"><displaynames><displayname description="Business Phone" languagecode="1033" /></displaynames></field><field name="address1_line1" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address 1: Street 1" languagecode="1033" /></displaynames></field><field name="isbackofficecustomer" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="No" languagecode="1033" /></labels></option><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option></options><displaynames><displayname description="Back Office Customer" languagecode="1033" /></displaynames></field><field name="address2_composite" requiredlevel="none" imemode="active" format="textarea" maxlength="1000"><displaynames><displayname description="Address 2" languagecode="1033" /></displaynames></field><field name="address2_county" requiredlevel="none" imemode="active" format="text" maxlength="50"><displaynames><displayname description="Address 2: County" languagecode="1033" /></displaynames></field><field name="shippingmethodcode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Shipping Method" languagecode="1033" /></displaynames></field><field name="anniversary" requiredlevel="none" imemode="inactive" format="date" Behavior="DateOnly"><displaynames><displayname description="Anniversary" languagecode="1033" /></displaynames></field><field name="ownerid" requiredlevel="systemrequired" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8,9"><displaynames><displayname description="Owner" languagecode="1033" /></displaynames></field><field name="address3_country" requiredlevel="none" imemode="active" format="text" maxlength="80"><displaynames><displayname description="Address3: Country/Region" languagecode="1033" /></displaynames></field><field name="haschildrencode" requiredlevel="none" defaultvalue="1"><options><option value="1"><labels><label description="Default Value" languagecode="1033" /></labels></option></options><displaynames><displayname description="Has Children" languagecode="1033" /></displaynames></field><field name="address2_fax" requiredlevel="none" imemode="inactive" format="text" maxlength="50"><displaynames><displayname description="Address 2: Fax" languagecode="1033" /></displaynames></field><field name="yomilastname" requiredlevel="none" imemode="active" format="phoneticguide" maxlength="150"><displaynames><displayname description="Yomi Last Name" languagecode="1033" /></displaynames></field><field name="createdon" requiredlevel="none" imemode="inactive" format="datetime" Behavior="Dynamic"><displaynames><displayname description="Created On" languagecode="1033" /></displaynames></field><field name="address1_line3" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address 1: Street 3" languagecode="1033" /></displaynames></field><field name="defaultpricelevelid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="1022"><displaynames><displayname description="Price List" languagecode="1033" /></displaynames></field><field name="slaid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="9750"><displaynames><displayname description="SLA" languagecode="1033" /></displaynames></field><field name="middlename" requiredlevel="none" imemode="active" format="text" maxlength="50"><displaynames><displayname description="Middle Name" languagecode="1033" /></displaynames></field><field name="address1_postalcode" requiredlevel="none" isglobalfilterenabled="true" issortableenabled="true" imemode="inactive" format="text" maxlength="20"><displaynames><displayname description="Address 1: ZIP/Postal Code" languagecode="1033" /></displaynames></field><field name="address3_line3" requiredlevel="none" imemode="active" format="text" maxlength="250"><displaynames><displayname description="Address3: Street 3" languagecode="1033" /></displaynames></field><field name="overriddencreatedon" requiredlevel="none" imemode="inactive" format="date" Behavior="Dynamic"><displaynames><displayname description="Record Created On" languagecode="1033" /></displaynames></field><field name="timespentbymeonemailandmeetings" requiredlevel="none" format="text" maxlength="1250"><displaynames><displayname description="Time Spent by me" languagecode="1033" /></displaynames></field></fields></entity><entity name="workflow" objecttypecode="4703"><fields><field name="deletestage" requiredlevel="none" defaultvalue="-1"><options><option value="20"><labels><label description="Pre-operation" languagecode="1033" /></labels></option><option value="40"><labels><label description="Post-operation" languagecode="1033" /></labels></option></options><displaynames><displayname description="Delete stage" languagecode="1033" /></displaynames></field><field name="name" requiredlevel="systemrequired" format="text" maxlength="100"><displaynames><displayname description="Process Name" languagecode="1033" /></displaynames></field><field name="createdonbehalfby" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Created By (Delegate)" languagecode="1033" /></displaynames></field><field name="scope" requiredlevel="systemrequired" defaultvalue="-1"><options><option value="1"><labels><label description="User" languagecode="1033" /></labels></option><option value="2"><labels><label description="Business Unit" languagecode="1033" /></labels></option><option value="3"><labels><label description="Parent: Child Business Units" languagecode="1033" /></labels></option><option value="4"><labels><label description="Organization" languagecode="1033" /></labels></option></options><displaynames><displayname description="Scope" languagecode="1033" /></displaynames></field><field name="inputparameters" requiredlevel="none" format="internalextentdata" maxlength="1073741823"><displaynames><displayname description="Input Parameters" languagecode="1033" /></displaynames></field><field name="modifiedby" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Modified By" languagecode="1033" /></displaynames></field><field name="runas" requiredlevel="none" defaultvalue="1"><options><option value="0"><labels><label description="Owner" languagecode="1033" /></labels></option><option value="1"><labels><label description="Calling User" languagecode="1033" /></labels></option></options><displaynames><displayname description="Run As User" languagecode="1033" /></displaynames></field><field name="ondemand" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="No" languagecode="1033" /></labels></option><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option></options><displaynames><displayname description="Run as On Demand" languagecode="1033" /></displaynames></field><field name="workflowid" requiredlevel="systemrequired"><displaynames><displayname description="Process" languagecode="1033" /></displaynames></field><field name="modifiedon" requiredlevel="none" imemode="inactive" format="datetime" Behavior="Dynamic"><displaynames><displayname description="Modified On" languagecode="1033" /></displaynames></field><field name="syncworkflowlogonfailure" requiredlevel="systemrequired" defaultvalue="0"><options><option value="0"><labels><label description="No" languagecode="1033" /></labels></option><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option></options><displaynames><displayname description="Log upon Failure" languagecode="1033" /></displaynames></field><field name="createdon" requiredlevel="none" imemode="inactive" format="datetime" Behavior="Dynamic"><displaynames><displayname description="Created On" languagecode="1033" /></displaynames></field><field name="uniquename" requiredlevel="none" format="text" maxlength="256"><displaynames><displayname description="Unique Name" languagecode="1033" /></displaynames></field><field name="businessprocesstype" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="Business Flow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Task Flow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Business Process Type" languagecode="1033" /></displaynames></field><field name="updatestage" requiredlevel="none" defaultvalue="-1"><options><option value="20"><labels><label description="Pre-operation" languagecode="1033" /></labels></option><option value="40"><labels><label description="Post-operation" languagecode="1033" /></labels></option></options><displaynames><displayname description="Update Stage" languagecode="1033" /></displaynames></field><field name="type" requiredlevel="systemrequired" defaultvalue="-1"><options><option value="1"><labels><label description="Definition" languagecode="1033" /></labels></option><option value="2"><labels><label description="Activation" languagecode="1033" /></labels></option><option value="3"><labels><label description="Template" languagecode="1033" /></labels></option></options><displaynames><displayname description="Type" languagecode="1033" /></displaynames></field><field name="owningbusinessunit" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="10"><displaynames><displayname description="Owning Business Unit" languagecode="1033" /></displaynames></field><field name="triggeroncreate" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="No" languagecode="1033" /></labels></option><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option></options><displaynames><displayname description="Trigger On Create" languagecode="1033" /></displaynames></field><field name="category" requiredlevel="systemrequired" defaultvalue="0"><options><option value="0"><labels><label description="Workflow" languagecode="1033" /></labels></option><option value="1"><labels><label description="Dialog" languagecode="1033" /></labels></option><option value="2"><labels><label description="Business Rule" languagecode="1033" /></labels></option><option value="3"><labels><label description="Action" languagecode="1033" /></labels></option><option value="4"><labels><label description="Business Process Flow" languagecode="1033" /></labels></option></options><displaynames><displayname description="Category" languagecode="1033" /></displaynames></field><field name="statecode" requiredlevel="systemrequired" defaultvalue="0"><states><state value="0" defaultstatus="1" invariantname="Draft"><labels><label description="Draft" languagecode="1033" /></labels></state><state value="1" defaultstatus="2" invariantname="Activated"><labels><label description="Activated" languagecode="1033" /></labels></state></states><displaynames><displayname description="Status" languagecode="1033" /></displaynames></field><field name="rendererobjecttypecode" requiredlevel="none" imemode="disabled" format="" minvalue="-2147483648" maxvalue="2147483647"><displaynames><displayname description="Renderer Type" languagecode="1033" /></displaynames></field><field name="asyncautodelete" requiredlevel="systemrequired" defaultvalue="0"><options><option value="0"><labels><label description="No" languagecode="1033" /></labels></option><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option></options><displaynames><displayname description="Delete Job On Completion" languagecode="1033" /></displaynames></field><field name="triggeronupdateattributelist" requiredlevel="none" format="internalextentdata" maxlength="1073741823"><displaynames><displayname description="Trigger On Update Attribute List" languagecode="1033" /></displaynames></field><field name="description" requiredlevel="none" format="textarea" maxlength="100000"><displaynames><displayname description="Description" languagecode="1033" /></displaynames></field><field name="statuscode" requiredlevel="none" defaultvalue="-1"><statuses><status value="1" state="0"><labels><label description="Draft" languagecode="1033" /></labels></status><status value="2" state="1"><labels><label description="Activated" languagecode="1033" /></labels></status></statuses><displaynames><displayname description="Status Reason" languagecode="1033" /></displaynames></field><field name="ismanaged" requiredlevel="systemrequired" defaultvalue="0"><options><option value="0"><labels><label description="Unmanaged" languagecode="1033" /></labels></option><option value="1"><labels><label description="Managed" languagecode="1033" /></labels></option></options><displaynames><displayname description="Is Managed" languagecode="1033" /></displaynames></field><field name="parentworkflowid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="4703"><displaynames><displayname description="Parent Process ID" languagecode="1033" /></displaynames></field><field name="createdby" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Created By" languagecode="1033" /></displaynames></field><field name="subprocess" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="No" languagecode="1033" /></labels></option><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option></options><displaynames><displayname description="Is Child Process" languagecode="1033" /></displaynames></field><field name="languagecode" requiredlevel="none" imemode="disabled" format="language" minvalue="0" maxvalue="2147483647"><displaynames><displayname description="Language" languagecode="1033" /></displaynames></field><field name="activeworkflowid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="4703"><displaynames><displayname description="Active Process ID" languagecode="1033" /></displaynames></field><field name="processorder" requiredlevel="none" format="" minvalue="0" maxvalue="2147483647"><displaynames><displayname description="Process Order" languagecode="1033" /></displaynames></field><field name="istransacted" requiredlevel="none" defaultvalue="1"><options><option value="0"><labels><label description="No" languagecode="1033" /></labels></option><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option></options><displaynames><displayname description="Is Transacted" languagecode="1033" /></displaynames></field><field name="modifiedonbehalfby" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Modified By (Delegate)" languagecode="1033" /></displaynames></field><field name="mode" requiredlevel="systemrequired" defaultvalue="0"><options><option value="0"><labels><label description="Background" languagecode="1033" /></labels></option><option value="1"><labels><label description="Real-time" languagecode="1033" /></labels></option></options><displaynames><displayname description="Mode" languagecode="1033" /></displaynames></field><field name="triggerondelete" requiredlevel="none" defaultvalue="0"><options><option value="0"><labels><label description="No" languagecode="1033" /></labels></option><option value="1"><labels><label description="Yes" languagecode="1033" /></labels></option></options><displaynames><displayname description="Trigger On Delete" languagecode="1033" /></displaynames></field><field name="createstage" requiredlevel="none" defaultvalue="-1"><options><option value="20"><labels><label description="Pre-operation" languagecode="1033" /></labels></option><option value="40"><labels><label description="Post-operation" languagecode="1033" /></labels></option></options><displaynames><displayname description="Create Stage" languagecode="1033" /></displaynames></field><field name="primaryentity" requiredlevel="systemrequired" defaultvalue="-1"><options /><displaynames><displayname description="Primary Entity" languagecode="1033" /></displaynames></field><field name="ownerid" requiredlevel="systemrequired" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="8"><displaynames><displayname description="Owner" languagecode="1033" /></displaynames></field><field name="sdkmessageid" requiredlevel="none" format="" lookupstyle="single" lookupbrowse="0" lookuptypes="4606"><displaynames><displayname description="SDK Message" languagecode="1033" /></displaynames></field></fields></entity></entities></propertiesxml><fieldxml><entities><entity name="account" otc="1"><fields><field name="accountcategorycode" description="Select a category to indicate whether the customer account is standard or preferred." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="accountclassificationcode" description="Select a classification code to indicate the potential value of the customer account based on the projected return on investment, cooperation level, sales cycle length or other criteria." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="accountid" description="Unique identifier of the account." requiredforplatform="true" requiredforgrid="true" requiredforform="false" validforadvancedfind="true" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="primarykey" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="accountnumber" description="Type an ID number or code for the account to quickly search and identify the account in system views." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="accountratingcode" description="Select a rating to indicate the value of the customer account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_addressid" description="Unique identifier for address 1." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="primarykey" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_addresstypecode" description="Select the primary address type." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_city" description="Type the city for the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="80" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_composite" description="Shows the complete primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_country" description="Type the country or region for the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="80" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_county" description="Type the county for the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_fax" description="Type the fax number associated with the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_freighttermscode" description="Select the freight terms for the primary address to make sure shipping orders are processed correctly." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_latitude" description="Type the latitude value for the primary address for use in mapping and other applications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="float" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_line1" description="Type the first line of the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_line2" description="Type the second line of the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_line3" description="Type the third line of the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_longitude" description="Type the longitude value for the primary address for use in mapping and other applications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="float" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_name" description="Type a descriptive name for the primary address, such as Corporate Headquarters." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="200" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_postalcode" description="Type the ZIP Code or postal code for the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_postofficebox" description="Type the post office box number of the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_primarycontactname" description="Type the name of the main contact at the accounts primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="150" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_shippingmethodcode" description="Select a shipping method for deliveries sent to this address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_stateorprovince" description="Type the state or province of the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_telephone1" description="Type the main phone number associated with the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_telephone2" description="Type a second phone number associated with the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_telephone3" description="Type a third phone number associated with the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_upszone" description="Type the UPS zone of the primary address to make sure shipping charges are calculated correctly and deliveries are made promptly, if shipped by UPS." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_utcoffset" description="Select the time zone, or UTC offset, for this address so that other people can reference it when they contact someone at this address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_addressid" description="Unique identifier for address 2." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="primarykey" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_addresstypecode" description="Select the secondary address type." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_city" description="Type the city for the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="80" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_composite" description="Shows the complete secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_country" description="Type the country or region for the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="80" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_county" description="Type the county for the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_fax" description="Type the fax number associated with the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_freighttermscode" description="Select the freight terms for the secondary address to make sure shipping orders are processed correctly." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_latitude" description="Type the latitude value for the secondary address for use in mapping and other applications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="float" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_line1" description="Type the first line of the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_line2" description="Type the second line of the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_line3" description="Type the third line of the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_longitude" description="Type the longitude value for the secondary address for use in mapping and other applications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="float" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_name" description="Type a descriptive name for the secondary address, such as Corporate Headquarters." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="200" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_postalcode" description="Type the ZIP Code or postal code for the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_postofficebox" description="Type the post office box number of the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_primarycontactname" description="Type the name of the main contact at the accounts secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="150" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_shippingmethodcode" description="Select a shipping method for deliveries sent to this address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_stateorprovince" description="Type the state or province of the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_telephone1" description="Type the main phone number associated with the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_telephone2" description="Type a second phone number associated with the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_telephone3" description="Type a third phone number associated with the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_upszone" description="Type the UPS zone of the secondary address to make sure shipping charges are calculated correctly and deliveries are made promptly, if shipped by UPS." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_utcoffset" description="Select the time zone, or UTC offset, for this address so that other people can reference it when they contact someone at this address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="aging30" description="For system use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="aging30_base" description="The base currency equivalent of the aging 30 field." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="aging60" description="For system use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="aging60_base" description="The base currency equivalent of the aging 60 field." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="aging90" description="For system use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="aging90_base" description="The base currency equivalent of the aging 90 field." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="businesstypecode" description="Select the legal designation or other business type of the account for contracts or reporting purposes." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="createdby" description="Shows who created the record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="createdbyexternalparty" description="Shows the external party who created the record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="3008" customfield="false" isnonsortablecalculatedfield="false" /><field name="createdon" description="Shows the date and time when the record was created. The date and time are displayed in the time zone selected in Microsoft Dynamics 365 options." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="createdonbehalfby" description="Shows who created the record on behalf of another user." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="creditlimit" description="Type the credit limit of the account. This is a useful reference when you address invoice and accounting issues with the customer." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="creditlimit_base" description="Shows the credit limit converted to the systems default base currency for reporting purposes." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="creditonhold" description="Select whether the credit for the account is on hold. This is a useful reference while addressing the invoice and accounting issues with the customer." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="customersizecode" description="Select the size category or range of the account for segmentation and reporting purposes." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="customertypecode" description="Select the category that best describes the relationship between the account and your organization." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="defaultpricelevelid" description="Choose the default price list associated with the account to make sure the correct product prices for this customer are applied in sales opportunities, quotes, and orders." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="1022" customfield="false" isnonsortablecalculatedfield="false" /><field name="description" description="Type additional information to describe the account, such as an excerpt from the companys website." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotbulkemail" description="Select whether the account allows bulk email sent through campaigns. If Do Not Allow is selected, the account can be added to marketing lists, but is excluded from email." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotbulkpostalmail" description="Select whether the account allows bulk postal mail sent through marketing campaigns or quick campaigns. If Do Not Allow is selected, the account can be added to marketing lists, but will be excluded from the postal mail." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotemail" description="Select whether the account allows direct email sent from Microsoft Dynamics 365." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotfax" description="Select whether the account allows faxes. If Do Not Allow is selected, the account will be excluded from fax activities distributed in marketing campaigns." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotphone" description="Select whether the account allows phone calls. If Do Not Allow is selected, the account will be excluded from phone call activities distributed in marketing campaigns." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotpostalmail" description="Select whether the account allows direct mail. If Do Not Allow is selected, the account will be excluded from letter activities distributed in marketing campaigns." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotsendmm" description="Select whether the account accepts marketing materials, such as brochures or catalogs." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="emailaddress1" description="Type the primary email address for the account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="emailaddress2" description="Type the secondary email address for the account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="emailaddress3" description="Type an alternate email address for the account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="entityimageid" description="For internal use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="uniqueidentifier" objecttypecode="1007" customfield="false" isnonsortablecalculatedfield="false" /><field name="exchangerate" description="Shows the conversion rate of the records currency. The exchange rate is used to convert all money fields in the record from the local currency to the systems default currency." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="9" datatype="decimal" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="fax" description="Type the fax number for the account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="followemail" description="Information about whether to allow following email activity like opens, attachment views and link clicks for emails sent to the account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="ftpsiteurl" description="Type the URL for the accounts FTP site to enable users to access data and share documents." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="200" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="importsequencenumber" description="Unique identifier of the data import or data migration that created this record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="industrycode" description="Select the accounts primary industry for use in marketing segmentation and demographic analysis." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="lastonholdtime" description="Contains the date and time stamp of the last on hold time." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="lastusedincampaign" description="Shows the date when the account was last included in a marketing campaign or quick campaign." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="marketcap" description="Type the market capitalization of the account to identify the companys equity, used as an indicator in financial performance analysis." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="marketcap_base" description="Shows the market capitalization converted to the systems default base currency." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="marketingonly" description="Whether is only for marketing" requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="masterid" description="Shows the master account that the account was merged with." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="1" customfield="false" isnonsortablecalculatedfield="false" /><field name="merged" description="Shows whether the account has been merged with another account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="modifiedby" description="Shows who last updated the record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="modifiedbyexternalparty" description="Shows the external party who modified the record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="3008" customfield="false" isnonsortablecalculatedfield="false" /><field name="modifiedon" description="Shows the date and time when the record was last updated. The date and time are displayed in the time zone selected in Microsoft Dynamics 365 options." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="modifiedonbehalfby" description="Shows who created the record on behalf of another user." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="name" description="Type the company or business name." requiredforplatform="false" requiredforgrid="true" requiredforform="true" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="160" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="numberofemployees" description="Type the number of employees that work at the account for use in marketing segmentation and demographic analysis." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="onholdtime" description="Shows how long, in minutes, that the record was on hold." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="opendeals" description="Number of open opportunities against an account and its child accounts." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="opendeals_date" description="Last Updated time of rollup field Open Deals." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="opendeals_state" description="State of rollup field Open Deals." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="openrevenue" description="Sum of open revenue against an account and its child accounts." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="openrevenue_base" description="Value of the Open Revenue in base currency." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="openrevenue_date" description="Last Updated time of rollup field Open Revenue." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="openrevenue_state" description="State of rollup field Open Revenue." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="originatingleadid" description="Shows the lead that the account was created from if the account was created by converting a lead in Microsoft Dynamics 365. This is used to relate the account to data on the originating lead for use in reporting and analytics." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="4" customfield="false" isnonsortablecalculatedfield="false" /><field name="overriddencreatedon" description="Date and time that the record was migrated." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="ownerid" description="Enter the user or team who is assigned to manage the record. This field is updated every time the record is assigned to a different user." requiredforplatform="true" requiredforgrid="false" requiredforform="true" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="owner" objecttypecode="7" customfield="false" isnonsortablecalculatedfield="false" /><field name="ownershipcode" description="Select the accounts ownership structure, such as public or private." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="owningbusinessunit" description="Shows the business unit that the record owner belongs to." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="10" customfield="false" isnonsortablecalculatedfield="false" /><field name="owningteam" description="Unique identifier of the team who owns the account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="9" customfield="false" isnonsortablecalculatedfield="false" /><field name="owninguser" description="Unique identifier of the user who owns the account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="parentaccountid" description="Choose the parent account associated with this account to show parent and child businesses in reporting and analytics." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="1" customfield="false" isnonsortablecalculatedfield="false" /><field name="participatesinworkflow" description="For system use only. Legacy Microsoft Dynamics CRM 3.0 workflow data." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="paymenttermscode" description="Select the payment terms to indicate when the customer needs to pay the total amount." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="preferredappointmentdaycode" description="Select the preferred day of the week for service appointments." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="preferredappointmenttimecode" description="Select the preferred time of day for service appointments." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="preferredcontactmethodcode" description="Select the preferred method of contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="preferredequipmentid" description="Choose the accounts preferred service facility or equipment to make sure services are scheduled correctly for the customer." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="4000" customfield="false" isnonsortablecalculatedfield="false" /><field name="preferredserviceid" description="Choose the accounts preferred service for reference when you schedule service activities." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="4001" customfield="false" isnonsortablecalculatedfield="false" /><field name="preferredsystemuserid" description="Choose the preferred service representative for reference when you schedule service activities for the account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="primarycontactid" description="Choose the primary contact for the account to provide quick access to contact details." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="2" customfield="false" isnonsortablecalculatedfield="false" /><field name="primarysatoriid" description="Primary Satori ID for Account" requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="200" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="primarytwitterid" description="Primary Twitter ID for Account" requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="128" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="processid" description="Shows the ID of the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="uniqueidentifier" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="revenue" description="Type the annual revenue for the account, used as an indicator in financial performance analysis." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="revenue_base" description="Shows the annual revenue converted to the systems default base currency. The calculations use the exchange rate specified in the Currencies area." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="sharesoutstanding" description="Type the number of shares available to the public for the account. This number is used as an indicator in financial performance analysis." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="shippingmethodcode" description="Select a shipping method for deliveries sent to the accounts address to designate the preferred carrier or other delivery option." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="sic" description="Type the Standard Industrial Classification (SIC) code that indicates the accounts primary industry of business, for use in marketing segmentation and demographic analysis." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="slaid" description="Choose the service level agreement (SLA) that you want to apply to the Account record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="9750" customfield="false" isnonsortablecalculatedfield="false" /><field name="slainvokedid" description="Last SLA that was applied to this case. This field is for internal use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="9750" customfield="false" isnonsortablecalculatedfield="false" /><field name="stageid" description="Shows the ID of the stage." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="uniqueidentifier" objecttypecode="4724" customfield="false" isnonsortablecalculatedfield="false" /><field name="statecode" description="Shows whether the account is active or inactive. Inactive accounts are read-only and cant be edited unless they are reactivated." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="state" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="statuscode" description="Select the accounts status." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="status" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="stockexchange" description="Type the stock exchange at which the account is listed to track their stock and financial performance of the company." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="telephone1" description="Type the main phone number for this account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="telephone2" description="Type a second phone number for this account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="telephone3" description="Type a third phone number for this account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="territorycode" description="Select a region or territory for the account for use in segmentation and analysis." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="territoryid" description="Choose the sales region or territory for the account to make sure the account is assigned to the correct representative and for use in segmentation and analysis." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="2013" customfield="false" isnonsortablecalculatedfield="false" /><field name="tickersymbol" description="Type the stock exchange symbol for the account to track financial performance of the company.You can click the code entered in this field to access the latest trading information from MSN Money." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="10" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="timespentbymeonemailandmeetings" description="Total time spent for emails (read and write) and meetings by me in relation to account record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="1250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="timezoneruleversionnumber" description="For internal use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="transactioncurrencyid" description="Choose the local currency for the record to make sure budgets are reported in the correct currency." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="9105" customfield="false" isnonsortablecalculatedfield="false" /><field name="traversedpath" description="For internal use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="utcconversiontimezonecode" description="Time zone code that was in use when the record was created." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="versionnumber" description="Version number of the account." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="unknown- data - type: timestamp" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="websiteurl" description="Type the accounts website URL to get quick details about the company profile." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="200" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="yominame" description="Type the phonetic spelling of the company name, if specified in Japanese, to make sure the name is pronounced correctly in phone calls and other communications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="160" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /></fields></entity><entity name="contact" otc="2"><fields><field name="accountrolecode" description="Select the contacts role within the company or sales process, such as decision maker, employee, or influencer." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_addressid" description="Unique identifier for address 1." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="primarykey" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_addresstypecode" description="Select the primary address type." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_city" description="Type the city for the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="80" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_composite" description="Shows the complete primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_country" description="Type the country or region for the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="80" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_county" description="Type the county for the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_fax" description="Type the fax number associated with the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_freighttermscode" description="Select the freight terms for the primary address to make sure shipping orders are processed correctly." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_latitude" description="Type the latitude value for the primary address for use in mapping and other applications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="float" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_line1" description="Type the first line of the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_line2" description="Type the second line of the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_line3" description="Type the third line of the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_longitude" description="Type the longitude value for the primary address for use in mapping and other applications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="float" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_name" description="Type a descriptive name for the primary address, such as Corporate Headquarters." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="200" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_postalcode" description="Type the ZIP Code or postal code for the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_postofficebox" description="Type the post office box number of the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_primarycontactname" description="Type the name of the main contact at the accounts primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="150" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_shippingmethodcode" description="Select a shipping method for deliveries sent to this address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_stateorprovince" description="Type the state or province of the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_telephone1" description="Type the main phone number associated with the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_telephone2" description="Type a second phone number associated with the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_telephone3" description="Type a third phone number associated with the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_upszone" description="Type the UPS zone of the primary address to make sure shipping charges are calculated correctly and deliveries are made promptly, if shipped by UPS." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address1_utcoffset" description="Select the time zone, or UTC offset, for this address so that other people can reference it when they contact someone at this address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_addressid" description="Unique identifier for address 2." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="primarykey" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_addresstypecode" description="Select the secondary address type." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_city" description="Type the city for the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="80" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_composite" description="Shows the complete secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_country" description="Type the country or region for the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="80" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_county" description="Type the county for the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_fax" description="Type the fax number associated with the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_freighttermscode" description="Select the freight terms for the secondary address to make sure shipping orders are processed correctly." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_latitude" description="Type the latitude value for the secondary address for use in mapping and other applications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="float" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_line1" description="Type the first line of the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_line2" description="Type the second line of the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_line3" description="Type the third line of the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_longitude" description="Type the longitude value for the secondary address for use in mapping and other applications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="float" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_name" description="Type a descriptive name for the secondary address, such as Corporate Headquarters." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="200" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_postalcode" description="Type the ZIP Code or postal code for the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_postofficebox" description="Type the post office box number of the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_primarycontactname" description="Type the name of the main contact at the accounts secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="150" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_shippingmethodcode" description="Select a shipping method for deliveries sent to this address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_stateorprovince" description="Type the state or province of the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_telephone1" description="Type the main phone number associated with the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_telephone2" description="Type a second phone number associated with the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_telephone3" description="Type a third phone number associated with the secondary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_upszone" description="Type the UPS zone of the secondary address to make sure shipping charges are calculated correctly and deliveries are made promptly, if shipped by UPS." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address2_utcoffset" description="Select the time zone, or UTC offset, for this address so that other people can reference it when they contact someone at this address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_addressid" description="Unique identifier for address 3." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="primarykey" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_addresstypecode" description="Select the third address type." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_city" description="Type the city for the 3rd address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="80" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_composite" description="Shows the complete third address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_country" description="the country or region for the 3rd address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="80" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_county" description="Type the county for the third address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_fax" description="Type the fax number associated with the third address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_freighttermscode" description="Select the freight terms for the third address to make sure shipping orders are processed correctly." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_latitude" description="Type the latitude value for the third address for use in mapping and other applications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="float" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_line1" description="the first line of the 3rd address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_line2" description="the second line of the 3rd address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_line3" description="the third line of the 3rd address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_longitude" description="Type the longitude value for the third address for use in mapping and other applications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="float" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_name" description="Type a descriptive name for the third address, such as Corporate Headquarters." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="200" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_postalcode" description="the ZIP Code or postal code for the 3rd address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_postofficebox" description="the post office box number of the 3rd address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="20" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_primarycontactname" description="Type the name of the main contact at the accounts third address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="150" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_shippingmethodcode" description="Select a shipping method for deliveries sent to this address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_stateorprovince" description="the state or province of the third address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_telephone1" description="Type the main phone number associated with the third address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_telephone2" description="Type a second phone number associated with the third address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_telephone3" description="Type a third phone number associated with the primary address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_upszone" description="Type the UPS zone of the third address to make sure shipping charges are calculated correctly and deliveries are made promptly, if shipped by UPS." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="address3_utcoffset" description="Select the time zone, or UTC offset, for this address so that other people can reference it when they contact someone at this address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="aging30" description="For system use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="aging30_base" description="Shows the Aging 30 field converted to the systems default base currency.The calculations use the exchange rate specified in the Currencies area." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="aging60" description="For system use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="aging60_base" description="Shows the Aging 60 field converted to the systems default base currency.The calculations use the exchange rate specified in the Currencies area." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="aging90" description="For system use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="aging90_base" description="Shows the Aging 90 field converted to the systems default base currency.The calculations use the exchange rate specified in the Currencies area." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="anniversary" description="Enter the date of the contacts wedding or service anniversary for use in customer gift programs or other communications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="annualincome" description="Type the contacts annual income for use in profiling and financial analysis." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="annualincome_base" description="Shows the Annual Income field converted to the systems default base currency.The calculations use the exchange rate specified in the Currencies area." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="assistantname" description="Type the name of the contacts assistant." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="assistantphone" description="Type the phone number for the contacts assistant." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="birthdate" description="Enter the contacts birthday for use in customer gift programs or other communications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="business2" description="Type a second business phone number for this contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="callback" description="Type a callback phone number for this contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="childrensnames" description="Type the names of the contacts children for reference in communications and client programs." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="255" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="company" description="Type the company phone of the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="contactid" description="Unique identifier of the contact." requiredforplatform="true" requiredforgrid="true" requiredforform="false" validforadvancedfind="true" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="primarykey" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="createdby" description="Shows who created the record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="createdbyexternalparty" description="Shows the external party who created the record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="3008" customfield="false" isnonsortablecalculatedfield="false" /><field name="createdon" description="Shows the date and time when the record was created.The date and time are displayed in the time zone selected in Microsoft Dynamics 365 options." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="createdonbehalfby" description="Shows who created the record on behalf of another user." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="creditlimit" description="Type the credit limit of the contact for reference when you address invoice and accounting issues with the customer." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="creditlimit_base" description="Shows the Credit Limit field converted to the systems default base currency for reporting purposes.The calculations use the exchange rate specified in the Currencies area." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="money" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="creditonhold" description="Select whether the contact is on a credit hold, for reference when addressing invoice and accounting issues." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="customersizecode" description="Select the size of the contacts company for segmentation and reporting purposes." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="customertypecode" description="Select the category that best describes the relationship between the contact and your organization." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="defaultpricelevelid" description="Choose the default price list associated with the contact to make sure the correct product prices for this customer are applied in sales opportunities, quotes, and orders." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="1022" customfield="false" isnonsortablecalculatedfield="false" /><field name="department" description="Type the department or business unit where the contact works in the parent company or business." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="description" description="Type additional information to describe the contact, such as an excerpt from the companys website." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotbulkemail" description="Select whether the contact accepts bulk email sent through marketing campaigns or quick campaigns.If Do Not Allow is selected, the contact can be added to marketing lists, but will be excluded from the email." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotbulkpostalmail" description="Select whether the contact accepts bulk postal mail sent through marketing campaigns or quick campaigns.If Do Not Allow is selected, the contact can be added to marketing lists, but will be excluded from the letters." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotemail" description="Select whether the contact allows direct email sent from Microsoft Dynamics 365. If Do Not Allow is selected, Microsoft Dynamics 365 will not send the email." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotfax" description="Select whether the contact allows faxes.If Do Not Allow is selected, the contact will be excluded from any fax activities distributed in marketing campaigns." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotphone" description="Select whether the contact accepts phone calls.If Do Not Allow is selected, the contact will be excluded from any phone call activities distributed in marketing campaigns." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotpostalmail" description="Select whether the contact allows direct mail.If Do Not Allow is selected, the contact will be excluded from letter activities distributed in marketing campaigns." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="donotsendmm" description="Select whether the contact accepts marketing materials, such as brochures or catalogs.Contacts that opt out can be excluded from marketing initiatives." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="educationcode" description="Select the contacts highest level of education for use in segmentation and analysis." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="emailaddress1" description="Type the primary email address for the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="emailaddress2" description="Type the secondary email address for the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="emailaddress3" description="Type an alternate email address for the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="employeeid" description="Type the employee ID or number for the contact for reference in orders, service cases, or other communications with the contacts organization." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="entityimageid" description="For internal use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="uniqueidentifier" objecttypecode="1007" customfield="false" isnonsortablecalculatedfield="false" /><field name="exchangerate" description="Shows the conversion rate of the records currency.The exchange rate is used to convert all money fields in the record from the local currency to the systems default currency." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="9" datatype="decimal" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="externaluseridentifier" description="Identifier for an external user." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="familystatuscode" description="Select the marital status of the contact for reference in follow - up phone calls and other communications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="fax" description="Type the fax number for the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="firstname" description="Type the contacts first name to make sure the contact is addressed correctly in sales calls, email, and marketing campaigns." requiredforplatform="false" requiredforgrid="false" requiredforform="true" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="followemail" description="Information about whether to allow following email activity like opens, attachment views and link clicks for emails sent to the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="ftpsiteurl" description="Type the URL for the contacts FTP site to enable users to access data and share documents." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="200" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="fullname" description="Combines and shows the contacts first and last names so that the full name can be displayed in views and reports." requiredforplatform="false" requiredforgrid="true" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="160" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="gendercode" description="Select the contacts gender to make sure the contact is addressed correctly in sales calls, email, and marketing campaigns." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="governmentid" description="Type the passport number or other government ID for the contact for use in documents or reports." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="haschildrencode" description="Select whether the contact has any children for reference in follow - up phone calls and other communications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="home2" description="Type a second home phone number for this contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="importsequencenumber" description="Unique identifier of the data import or data migration that created this record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="isbackofficecustomer" description="Select whether the contact exists in a separate accounting or other system, such as Microsoft Dynamics GP or another ERP database, for use in integration processes." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="jobtitle" description="Type the job title of the contact to make sure the contact is addressed correctly in sales calls, email, and marketing campaigns." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="lastname" description="Type the contacts last name to make sure the contact is addressed correctly in sales calls, email, and marketing campaigns." requiredforplatform="false" requiredforgrid="false" requiredforform="true" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="lastonholdtime" description="Contains the date and time stamp of the last on hold time." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="lastusedincampaign" description="Shows the date when the contact was last included in a marketing campaign or quick campaign." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="true" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="leadsourcecode" description="Select the primary marketing source that directed the contact to your organization." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="managername" description="Type the name of the contacts manager for use in escalating issues or other follow- up communications with the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="managerphone" description="Type the phone number for the contacts manager." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="marketingonly" description="Whether is only for marketing" requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="masterid" description="Unique identifier of the master contact for merge." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="2" customfield="false" isnonsortablecalculatedfield="false" /><field name="merged" description="Shows whether the account has been merged with a master contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="middlename" description="Type the contacts middle name or initial to make sure the contact is addressed correctly." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="mobilephone" description="Type the mobile phone number for the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="modifiedby" description="Shows who last updated the record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="modifiedbyexternalparty" description="Shows the external party who modified the record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="3008" customfield="false" isnonsortablecalculatedfield="false" /><field name="modifiedon" description="Shows the date and time when the record was last updated.The date and time are displayed in the time zone selected in Microsoft Dynamics 365 options." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="modifiedonbehalfby" description="Shows who last updated the record on behalf of another user." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="nickname" description="Type the contacts nickname." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="numberofchildren" description="Type the number of children the contact has for reference in follow - up phone calls and other communications." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="onholdtime" description="Shows how long, in minutes, that the record was on hold." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="originatingleadid" description="Shows the lead that the contact was created if the contact was created by converting a lead in Microsoft Dynamics 365. This is used to relate the contact to the data on the originating lead for use in reporting and analytics." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="4" customfield="false" isnonsortablecalculatedfield="false" /><field name="overriddencreatedon" description="Date and time that the record was migrated." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="ownerid" description="Enter the user or team who is assigned to manage the record.This field is updated every time the record is assigned to a different user." requiredforplatform="true" requiredforgrid="false" requiredforform="true" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="owner" objecttypecode="7" customfield="false" isnonsortablecalculatedfield="false" /><field name="owningbusinessunit" description="Unique identifier of the business unit that owns the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="10" customfield="false" isnonsortablecalculatedfield="false" /><field name="owningteam" description="Unique identifier of the team who owns the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="9" customfield="false" isnonsortablecalculatedfield="false" /><field name="owninguser" description="Unique identifier of the user who owns the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="pager" description="Type the pager number for the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="parentcustomerid" description="Select the parent account or parent contact for the contact to provide a quick link to additional details, such as financial information, activities, and opportunities." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="customer" objecttypecode="1" customfield="false" isnonsortablecalculatedfield="false" /><field name="parentcustomerid" description="Select the parent account or parent contact for the contact to provide a quick link to additional details, such as financial information, activities, and opportunities." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="customer" objecttypecode="2" customfield="false" isnonsortablecalculatedfield="false" /><field name="participatesinworkflow" description="Shows whether the contact participates in workflow rules." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="paymenttermscode" description="Select the payment terms to indicate when the customer needs to pay the total amount." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="preferredappointmentdaycode" description="Select the preferred day of the week for service appointments." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="preferredappointmenttimecode" description="Select the preferred time of day for service appointments." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="preferredcontactmethodcode" description="Select the preferred method of contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="preferredequipmentid" description="Choose the contacts preferred service facility or equipment to make sure services are scheduled correctly for the customer." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="4000" customfield="false" isnonsortablecalculatedfield="false" /><field name="preferredserviceid" description="Choose the contacts preferred service to make sure services are scheduled correctly for the customer." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="4001" customfield="false" isnonsortablecalculatedfield="false" /><field name="preferredsystemuserid" description="Choose the regular or preferred customer service representative for reference when scheduling service activities for the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="processid" description="Shows the ID of the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="uniqueidentifier" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="salutation" description="Type the salutation of the contact to make sure the contact is addressed correctly in sales calls, email messages, and marketing campaigns." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="shippingmethodcode" description="Select a shipping method for deliveries sent to this address." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="slaid" description="Choose the service level agreement (SLA) that you want to apply to the Contact record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="9750" customfield="false" isnonsortablecalculatedfield="false" /><field name="slainvokedid" description="Last SLA that was applied to this case. This field is for internal use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="9750" customfield="false" isnonsortablecalculatedfield="false" /><field name="spousesname" description="Type the name of the contacts spouse or partner for reference during calls, events, or other communications with the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="stageid" description="Shows the ID of the stage." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="uniqueidentifier" objecttypecode="4724" customfield="false" isnonsortablecalculatedfield="false" /><field name="statecode" description="Shows whether the contact is active or inactive.Inactive contacts are read- only and cant be edited unless they are reactivated." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="state" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="statuscode" description="Select the contacts status." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="status" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="subscriptionid" description="For internal use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="false" validforreadapi="false" maxlength="16" datatype="uniqueidentifier" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="suffix" description="Type the suffix used in the contacts name, such as Jr. or Sr. to make sure the contact is addressed correctly in sales calls, email, and marketing campaigns." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="10" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="telephone1" description="Type the main phone number for this contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="telephone2" description="Type a second phone number for this contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="telephone3" description="Type a third phone number for this contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="50" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="territorycode" description="Select a region or territory for the contact for use in segmentation and analysis." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="timespentbymeonemailandmeetings" description="Total time spent for emails (read and write) and meetings by me in relation to the contact record." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="true" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="1250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="timezoneruleversionnumber" description="For internal use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="transactioncurrencyid" description="Choose the local currency for the record to make sure budgets are reported in the correct currency." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="9105" customfield="false" isnonsortablecalculatedfield="false" /><field name="traversedpath" description="For internal use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1250" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="utcconversiontimezonecode" description="Time zone code that was in use when the record was created." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="versionnumber" description="Version number of the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="unknown-data-type: timestamp" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="websiteurl" description="Type the contacts professional or personal website or blog URL." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="200" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="yomifirstname" description="Type the phonetic spelling of the contacts first name, if the name is specified in Japanese, to make sure the name is pronounced correctly in phone calls with the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="150" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="yomifullname" description="Shows the combined Yomi first and last names of the contact so that the full phonetic name can be displayed in views and reports." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="450" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="yomilastname" description="Type the phonetic spelling of the contacts last name, if the name is specified in Japanese, to make sure the name is pronounced correctly in phone calls with the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="150" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="yomimiddlename" description="Type the phonetic spelling of the contacts middle name, if the name is specified in Japanese, to make sure the name is pronounced correctly in phone calls with the contact." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="150" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /></fields></entity><entity name="workflow" otc="4703"><fields><field name="activeworkflowid" description="Unique identifier of the latest activation record for the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="4703" customfield="false" isnonsortablecalculatedfield="false" /><field name="activities" description="Activities that are part of the business logic of the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="asyncautodelete" description="Indicates whether the asynchronous system job is automatically deleted on completion." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="businessprocesstype" description="Business Process Type." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="category" description="Category of the process." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="clientdata" description="Business logic converted into client data" requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="componentstate" description="For internal use only." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="createdby" description="Unique identifier of the user who created the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="createdon" description="Date and time when the process was created." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="createdonbehalfby" description="Unique identifier of the delegate user who created the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="createstage" description="Stage of the process when triggered on Create." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="deletestage" description="Stage of the process when triggered on Delete." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="description" description="Description of the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="entityimageid" description="For internal use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="uniqueidentifier" objecttypecode="1007" customfield="false" isnonsortablecalculatedfield="false" /><field name="formid" description="Unique identifier of the associated form." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="uniqueidentifier" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="inputparameters" description="Input parameters to the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="introducedversion" description="Version in which the form is introduced." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="48" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="iscrmuiworkflow" description="Indicates whether the process was created using the Microsoft Dynamics 365 Web application." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="iscustomizable" description="Information that specifies whether this component can be customized." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="unknown-data-type: managedproperty" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="ismanaged" description="Indicates whether the solution component is part of a managed solution." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="istransacted" description="Whether or not the steps in the process are executed in a single transaction." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="languagecode" description="Language of the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="mode" description="Shows the mode of the process." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="modifiedby" description="Unique identifier of the user who last modified the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="modifiedon" description="Date and time when the process was last modified." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="modifiedonbehalfby" description="Unique identifier of the delegate user who last modified the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="name" description="Name of the process." requiredforplatform="true" requiredforgrid="true" requiredforform="true" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="100" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="ondemand" description="Indicates whether the process is able to run as an on-demand process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="overwritetime" description="For internal use only." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="datetime" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="ownerid" description="Unique identifier of the user or team who owns the process." requiredforplatform="true" requiredforgrid="false" requiredforform="true" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="16" datatype="owner" objecttypecode="7" customfield="false" isnonsortablecalculatedfield="false" /><field name="owningbusinessunit" description="Unique identifier of the business unit that owns the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="10" customfield="false" isnonsortablecalculatedfield="false" /><field name="owningteam" description="Unique identifier of the team who owns the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="9" customfield="false" isnonsortablecalculatedfield="false" /><field name="owninguser" description="Unique identifier of the user who owns the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="8" customfield="false" isnonsortablecalculatedfield="false" /><field name="parentworkflowid" description="Unique identifier of the definition for process activation." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="4703" customfield="false" isnonsortablecalculatedfield="false" /><field name="plugintypeid" description="Unique identifier of the plug-in type." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="primaryentity" description="Primary entity for the process. The process can be associated for one or more SDK operations defined on the primary entity." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="processorder" description="Type the business process flow order." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="processroleassignment" description="Contains the role assignment for the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="rank" description="Indicates the rank for order of execution for the synchronous workflow." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="rendererobjecttypecode" description="The renderer type of Workflow" requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="integer" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="rules" description="Rules that define business logic in the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="runas" description="Specifies the system user account under which a workflow executes." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="scope" description="Scope of the process." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="sdkmessageid" description="Unique identifier of the SDK Message associated with this workflow." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="lookup" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="solutionid" description="Unique identifier of the associated solution." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="uniqueidentifier" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="statecode" description="Status of the process." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="false" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="state" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="statuscode" description="Additional information about status of the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="status" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="subprocess" description="Indicates whether the process can be included in other processes as a child process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="syncworkflowlogonfailure" description="Select whether synchronous workflow failures will be saved to log files." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="triggeroncreate" description="Indicates whether the process will be triggered when the primary entity is created." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="triggerondelete" description="Indicates whether the process will be triggered on deletion of the primary entity." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="1" datatype="boolean" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="triggeronupdateattributelist" description="Attributes that trigger the process when updated." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="type" description="Type of the process." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="uidata" description="For internal use only." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="uniquename" description="Unique name of the process" requiredforplatform="false" requiredforgrid="true" requiredforform="true" validforadvancedfind="true" validforgrid="true" validforform="true" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="256" datatype="text" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="updatestage" description="Select the stage a process will be triggered on update." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="true" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="4" datatype="picklist" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="versionnumber" requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="8" datatype="unknown-data-type: timestamp" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="workflowid" description="Unique identifier of the process." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="true" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="primarykey" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="workflowidunique" description="For internal use only." requiredforplatform="true" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="false" validforupdateapi="false" validforreadapi="true" maxlength="16" datatype="uniqueidentifier" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /><field name="xaml" description="XAML that defines the process." requiredforplatform="false" requiredforgrid="false" requiredforform="false" validforadvancedfind="false" validforgrid="false" validforform="false" validforcreateapi="true" validforupdateapi="true" validforreadapi="true" maxlength="5000" datatype="memo" objecttypecode="0" customfield="false" isnonsortablecalculatedfield="false" /></fields></entity></entities></fieldxml></fieldpropertiesxml>';
            ExportToExcel.Constants = Constants;
        })(ExportToExcel = AppCommon.ExportToExcel || (AppCommon.ExportToExcel = {}));
    })(AppCommon = MscrmControls.AppCommon || (MscrmControls.AppCommon = {}));
})(MscrmControls || (MscrmControls = {}));
/**
 * This code was generated from a script.
 * Manual changes to this file will be overwritten if the code is regenerated.
 *
 * @license Copyright (c) Microsoft Corporation. All rights reserved.
 */
/**
* @license Copyright (c) Microsoft Corporation.  All rights reserved.
*/
/// <reference path="../../../../TypeDefinitions/mscrm.d.ts" />
/// <reference path="CommonReferences.ts" />
/// <reference path="../../../../../references/external/TypeDefinitions/jquery.d.ts" /> 
/**
* @license Copyright (c) Microsoft Corporation.  All rights reserved.
*/
/// <reference path="privatereferences.ts"/>
var MscrmControls;
(function (MscrmControls) {
    var AppCommon;
    (function (AppCommon) {
        var ExportToExcel;
        (function (ExportToExcel) {
            'use strict';
            var entitSelectorOptionSetKey = "entitSelectorOptionSetKey";
            var entitSelectorOptionSetId = "entitSelectorOptionSetId";
            var ExportToExcelControl = (function () {
                /**
                 * Empty constructor.
                 */
                function ExportToExcelControl() {
                    var _this = this;
                    this._entityTypeCode = 0;
                    this._entitiesSelectorOptions = [];
                    this._entitiesDataObject = [];
                    this._entitiesFieldsDataObj = [];
                    this._relIdOfCurrentEntity = "";
                    this._selectedEntity = "";
                    this._typeNameDictionary = {};
                    this._isInitialEntityDataFetched = false;
                    this._isInitialFieldPropertiesDataFetched = false;
                    this._entityLayoutXml = "";
                    this._processOnlyForInitialFetch = false;
                    this.sortType = 'asc';
                    this.clearTimer = function () {
                        if (_this.timer) {
                            clearTimeout(_this.timer);
                            _this.timer = null;
                        }
                    };
                    this._applyStyles = null;
                }
                /**
                 * This function should be used for any initial setup necessary for your control.
                 * @params context The "Input Bag" containing the parameters and other control metadata.
                 * @params notifyOutputchanged The method for this control to notify the framework that it has new outputs
                 * @params state The user state for this control set from setState in the last session
                 * @params container The div element to draw this control in
                 */
                ExportToExcelControl.prototype.init = function (context, notifyOutputChanged, state) {
                    // custom code goes here
                    this._context = context;
                    this._entityFetchXml = this._context.parameters.fetchxml ? this._context.parameters.fetchxml.raw : null;
                    this._entityLayoutXml = this._context.parameters.layoutxml ? this._context.parameters.layoutxml.raw : null;
                    this._entityTypeCode = this._context.parameters.typecode ? this._context.parameters.typecode.raw : null;
                    this._exportType = this._context.parameters.exporttype ? this._context.parameters.exporttype.raw : null;
                    this._applyStyles = new ExportToExcel.ExportToExcelStyles(context);
                    this._updatedFetchXml = this._entityFetchXml;
                    ExportToExcelControl.EntityFieldsSortType = ExportToExcel.EntityFieldsSortByField.DisplayName;
                    this._notifyOutputChanged = notifyOutputChanged || (function () { });
                    var excelControlInitEvent = new ExportToExcel.ExcelControlInitEvent(this._entityTypeCode, this._exportType == 2 ? "Dynamic" : "Pivot");
                    Xrm.Reporting.reportEvent(excelControlInitEvent);
                };
                /**
                 * This function will recieve an "Input Bag" containing the values currently assigned to the parameters in your manifest
                 * It will send down the latest values (static or dynamic) that are assigned as defined by the manifest & customization experience
                 * as well as resource, client, and theming info (see mscrm.d.ts)
                 * @params context The "Input Bag" as described above
                 */
                ExportToExcelControl.prototype.updateView = function (context) {
                    var _this = this;
                    // custom code goes here
                    var controlsList = [];
                    if (!this._isInitialEntityDataFetched) {
                        this.processEntityXmlAndCreateOptions();
                    }
                    if (this._isInitialEntityDataFetched) {
                        controlsList.push(this.renderEntitySelector());
                        controlsList.push(this.createGridHeaderElements());
                        if (this.entitySelectorExists()) {
                            controlsList.push(this.renderAttributesGrid());
                        }
                        else {
                            this.clearTimer();
                            this.timer = setTimeout(function () { return _this.renderControl(true); }, 150);
                        }
                    }
                    $('[data-id="tabpanel-{22afcbc7-1e04-4b2a-9e51-7dcaee23c7c8}"]').css({ "overflow": "hidden" });
                    return context.factory.createElement("CONTAINER", {
                        style: {
                            key: "maincontatiner",
                            id: "maincontatiner",
                            display: "flex",
                            flexDirection: "column",
                            width: "100%"
                        }
                    }, controlsList);
                };
                /**
                 * This function will return an "Output Bag" to the Crm Infrastructure
                 * The ouputs will contain a value for each property marked as "input-output"/"bound" in your manifest
                 * i.e. if your manifest has a property "value" that is an "input-output", and you want to set that to the local variable "myvalue" you should return:
                 * {
                 *		value: myvalue
                 * };
                 * @returns The "Output Bag" containing values to pass to the infrastructure
                 */
                ExportToExcelControl.prototype.getOutputs = function () {
                    var result = {
                        fetchxml: this._updatedFetchXml.toString(),
                        layoutxml: this._entityLayoutXml.toString()
                    };
                    return result;
                };
                /**
                 * This function will be called when the control is destroyed
                 * It should be used for cleanup and releasing any memory the control is using
                 */
                ExportToExcelControl.prototype.destroy = function () {
                };
                ExportToExcelControl.prototype.entitySelectorExists = function () {
                    this._heightEntitySelector = jQuery("*").find("div[data-id='entitySelector_id.fieldControl-entitySelectorContainer']").outerHeight();
                    this._heightgridHeader = jQuery("*").find("div[data-id='entitySelector_id.fieldControl-attributesGridHeaderContainer1']").outerHeight();
                    return (null == this._heightEntitySelector) ? false : true;
                };
                ExportToExcelControl.prototype.retrieveEntitiesXml = function () {
                    var getEntitiesSdkResultString = ExportToExcel.ExportControlUtils.GetExportEntitiesXml(this._entityTypeCode, this._entityFetchXml, this._context); //Constants.EntitiesXml;
                    return getEntitiesSdkResultString;
                };
                ExportToExcelControl.prototype.retrieveFeildAndPropertiesXml = function (fetchXml) {
                    var fieldAndPropertiesXml = ExportToExcel.ExportControlUtils.GetFieldAndPropertiesXml(this._entityTypeCode, fetchXml, this._context); //Constants.FieldAndPropertiesXml;
                    return fieldAndPropertiesXml;
                };
                ExportToExcelControl.prototype.renderEntitySelector = function () {
                    var entitySelectorContainer;
                    var entitySelectorContainerElements = [];
                    var recordTypeLabel = this._context.factory.createElement("LABEL", {
                        id: "header_label",
                        key: "header_label",
                        style: {
                            fontFamily: this._context.theming.fontfamilies.semibold,
                            fontSize: this._context.theming.fontsizes.font100,
                            color: this._context.theming.colors.grays.gray06,
                            paddingLeft: this._context.theming.measures.measure075,
                            paddingRight: this._context.theming.measures.measure075,
                            overflow: "hidden",
                            textOverflow: "ellipsis",
                            whiteSpace: "nowrap",
                            display: "inline-block",
                            flex: "1 1 auto"
                        }
                    }, this._context.resources.getString("SystemCustomization.RecordType"));
                    entitySelectorContainerElements.push(recordTypeLabel);
                    if (this._currentSelectedOption == null) {
                        this._currentSelectedOption = this._entitiesSelectorOptions[0];
                    }
                    this._entitiesSelectorOptions = ExportToExcel.ExportControlUtils.sortOnLabel(this._entitiesSelectorOptions);
                    entitySelectorContainerElements.push(this.CreateOptionSetControl(this._context, entitSelectorOptionSetKey, entitSelectorOptionSetId, this._entitiesSelectorOptions));
                    entitySelectorContainer = this._context.factory.createElement("CONTAINER", {
                        key: "entitySelectorContainer",
                        id: "entitySelectorContainer",
                        accessibilityLabel: this._context.resources.getString("SystemCustomization.RecordType"),
                        style: {
                            width: "100%",
                            display: "flex",
                            flexDirection: "row",
                            justifyContent: "space-between",
                            alignContent: "center",
                            alignItems: "center",
                            paddingBottom: "0.5rem"
                        }
                    }, entitySelectorContainerElements);
                    return entitySelectorContainer;
                };
                ExportToExcelControl.prototype.renderAttributesGrid = function () {
                    var attributesGridContainer;
                    var attributesGridContainerElements = [];
                    // attributesGridContainerElements.push(this.createGridHeaderElements());
                    attributesGridContainerElements.push(this.renderGridRows());
                    var dialogHeight = jQuery("*").find("div[id^='dialogContentContainer']").height();
                    var dialogHeaderHeight = jQuery("*").find("div[data-id='dialogHeader']").outerHeight();
                    var dialogFooterHeight = jQuery("*").find("div[data-id='dialogFooter']").outerHeight();
                    var topMargin = 10; //marginTop: 0.5rem
                    var height = dialogHeight - dialogHeaderHeight - this._heightEntitySelector - this._heightgridHeader - dialogFooterHeight - topMargin;
                    attributesGridContainer = this._context.factory.createElement("CONTAINER", {
                        key: "attributesGridContainer",
                        id: "attributesGridContainer",
                        style: this._applyStyles.GetGridContainer(height.toString() + 'px')
                    }, attributesGridContainerElements);
                    return attributesGridContainer;
                };
                ExportToExcelControl.prototype.createGridHeaderElements = function () {
                    var attributesGridHeaderContainer;
                    var attributesGridHeaderContainerElements = [];
                    // Create the checkbox element
                    var selectAllCheckBoxHeaderElement = this._context.factory.createElement("BOOLEAN", {
                        type: "checkbox",
                        id: 'selectAllCheckBoxHeaderElementKey',
                        key: 'selectAllCheckBoxHeaderElementId',
                        tagname: "INPUT",
                        style: this._applyStyles.GetHeaderCheckboxContainer(),
                        onValueChange: null,
                        value: false
                    });
                    // Create the Check all elements container
                    var selectAllCheckBoxHeaderElementConatiner = this._context.factory.createElement("CONTAINER", {
                        key: "selectAllCheckBoxElementConatiner",
                        id: "selectAllCheckBoxElementConatiner",
                        style: this._applyStyles.GetHeaderCheckbox()
                    }, selectAllCheckBoxHeaderElement);
                    var displayNameImage = this._context.factory.createElement("MICROSOFTICON", {
                        id: "displayNameImage_id",
                        key: "displayNameImage_key",
                        type: (this.sortType === 'asc') ? 58 : 59,
                        style: (ExportToExcelControl.EntityFieldsSortType === ExportToExcel.EntityFieldsSortByField.DisplayName) ? this._applyStyles.GetArrowStyles() : this._applyStyles.GetArrowStylesNone()
                    });
                    var displayNameHeaderArrow = this._context.factory.createElement("CONTAINER", {
                        id: "displayNameHeaderArrow_id",
                        key: "displayNameHeaderArrow_key",
                        accessibilityHidden: true,
                    }, displayNameImage);
                    var nameImage = this._context.factory.createElement("MICROSOFTICON", {
                        id: "nameImage_id",
                        key: "nameImage_key",
                        type: (this.sortType === 'asc') ? 58 : 59,
                        style: (ExportToExcelControl.EntityFieldsSortType === ExportToExcel.EntityFieldsSortByField.Name) ? this._applyStyles.GetArrowStyles() : this._applyStyles.GetArrowStylesNone()
                    });
                    var nameHeaderArrow = this._context.factory.createElement("CONTAINER", {
                        id: "nameHeaderArrow_id",
                        key: "nameHeaderArrow_key",
                        accessibilityHidden: true,
                    }, nameImage);
                    var typeImage = this._context.factory.createElement("MICROSOFTICON", {
                        id: "typeImage_id",
                        key: "typeImage_key",
                        type: (this.sortType === 'asc') ? 58 : 59,
                        style: (ExportToExcelControl.EntityFieldsSortType === ExportToExcel.EntityFieldsSortByField.DataType) ? this._applyStyles.GetArrowStyles() : this._applyStyles.GetArrowStylesNone()
                    });
                    var typeHeaderArrow = this._context.factory.createElement("CONTAINER", {
                        id: "typeHeaderArrow_id",
                        key: "typeHeaderArrow_key",
                        accessibilityHidden: true,
                    }, typeImage);
                    // Create the Display Name sort button element
                    var displayNameSortButtonHeaderElement = this._context.factory.createElement("BUTTON", {
                        key: "displayNameSortButtonHeaderElementKey",
                        id: "displayNameSortButtonHeaderElementId",
                        style: this._applyStyles.GetDisplayNameHeaderCellComponent(),
                        hidden: false,
                        accessibilityLabel: this._context.resources.getString("SystemCustomization.Sort") + this._context.resources.getString("SystemCustomization.DisplayName"),
                        onClick: this.displayNameSortButtonClickHandler.bind(this)
                    }, this._context.resources.getString("SystemCustomization.DisplayName"));
                    // Create the Display Name sort button element container
                    var displayNameSortButtonHeaderElementConatiner = this._context.factory.createElement("CONTAINER", {
                        key: "displayNameSortButtonHeaderElementConatiner",
                        id: "displayNameSortButtonHeaderElementConatiner",
                        style: this._applyStyles.GetHeaderCellContainer()
                    }, [displayNameSortButtonHeaderElement, displayNameHeaderArrow]);
                    attributesGridHeaderContainerElements.push(displayNameSortButtonHeaderElementConatiner);
                    // Create the Name sort button element
                    var nameSortButtonHeaderElement = this._context.factory.createElement("BUTTON", {
                        key: "nameSortButtonHeaderElementKey",
                        id: "nameSortButtonHeaderElementId",
                        style: this._applyStyles.GetNameHeaderCellComponent(),
                        hidden: false,
                        accessibilityLabel: this._context.resources.getString("SystemCustomization.Sort") + this._context.resources.getString("SystemCustomization.Name"),
                        onClick: this.nameSortButtonClickHandler.bind(this)
                    }, this._context.resources.getString("SystemCustomization.Name"));
                    // Create the Name sort button element container
                    var nameSortButtonHeaderElementConatiner = this._context.factory.createElement("CONTAINER", {
                        key: "nameSortButtonElementKeyContainer",
                        id: "nameSortButtonElementKeyContainer",
                        style: this._applyStyles.GetHeaderCellContainer(),
                    }, [nameSortButtonHeaderElement, nameHeaderArrow]);
                    attributesGridHeaderContainerElements.push(nameSortButtonHeaderElementConatiner);
                    // Create the Type sort button element
                    var typeSortButtonHeaderElement = this._context.factory.createElement("BUTTON", {
                        key: "typeSortButtonHeaderElementKey",
                        id: "typeSortButtonHeaderElementId",
                        style: this._applyStyles.GetTypeHeaderCellComponent(),
                        hidden: false,
                        accessibilityLabel: this._context.resources.getString("SystemCustomization.Sort") + this._context.resources.getString("SystemCustomization.Type"),
                        onClick: this.dataTypeSortButtonClickHandler.bind(this)
                    }, this._context.resources.getString("SystemCustomization.Type"));
                    // Create the Type sort button element container
                    var typeSortButtonHeaderElementConatiner = this._context.factory.createElement("CONTAINER", {
                        key: "typeSortButtonHeaderElementKeyContainer",
                        id: "typeSortButtonHeaderElementKeyContainer",
                        style: this._applyStyles.GetHeaderCellContainer(),
                    }, [typeSortButtonHeaderElement, typeHeaderArrow]);
                    attributesGridHeaderContainerElements.push(typeSortButtonHeaderElementConatiner);
                    // Create the container for the button elements
                    var headerRowContainerWithoutCheckbox = this._context.factory.createElement("CONTAINER", {
                        key: "headerRowContainerWithoutCheckbox",
                        id: "headerRowContainerWithoutCheckbox",
                        style: this._applyStyles.GetHeaderRowElementsContainer()
                    }, attributesGridHeaderContainerElements);
                    // Create the container for all the header elements
                    var attributesGridHeaderContainer1 = this._context.factory.createElement("CONTAINER", {
                        key: "attributesGridHeaderContainer1",
                        id: "attributesGridHeaderContainer1",
                        style: this._applyStyles.GetHeaderContainer()
                    }, [selectAllCheckBoxHeaderElementConatiner, headerRowContainerWithoutCheckbox]);
                    // Create the container for all the header elements
                    attributesGridHeaderContainer = this._context.factory.createElement("CONTAINER", {
                        key: "attributesGridHeaderContainer",
                        id: "attributesGridHeaderContainer",
                        style: {
                            width: "100%"
                        }
                    }, attributesGridHeaderContainer1);
                    return attributesGridHeaderContainer;
                };
                ExportToExcelControl.prototype.renderGridRows = function () {
                    var attributesGridDataContainer;
                    var attributesGridDataContainerElements = [];
                    var entityFieldsDataObj = this.retrieveEntityFieldsDataObj(this._selectedEntity);
                    for (var i = 0; i < entityFieldsDataObj.entityAttributesList.length; i++) {
                        attributesGridDataContainerElements.push(this.createGridRowElements(entityFieldsDataObj.entityAttributesList[i]));
                    }
                    attributesGridDataContainer = this._context.factory.createElement("CONTAINER", {
                        key: "attributesGridDataContainer",
                        id: "attributesGridDataContainer",
                        style: {
                            display: "grid",
                            flexDirection: "column",
                            width: "100%"
                        }
                    }, attributesGridDataContainerElements);
                    return attributesGridDataContainer;
                };
                ExportToExcelControl.prototype.createGridRowElements = function (fieldRecord) {
                    var viewPortWidth = this.getViewPortWidth().w;
                    if (viewPortWidth < 10) {
                        return this.renderGridRowElementsForLowerViewPort(fieldRecord);
                    }
                    else {
                        return this.renderGridRowElementsForNormalViewPort(fieldRecord);
                    }
                };
                ExportToExcelControl.prototype.renderGridRowElementsForLowerViewPort = function (fieldRecord) {
                    var attributesGridRowContainer;
                    var attributesGridRowContainerElements = [];
                    var attributename = fieldRecord.name;
                    var suffixfText = this._selectedEntity + attributename;
                    // Create the checkbox element
                    var selectAllCheckBoxElement = this._context.factory.createElement("BOOLEAN", {
                        type: "checkbox",
                        role: "checkbox",
                        tabIndex: 0,
                        id: 'selectAllCheckBoxElementKey' + suffixfText,
                        key: 'selectAllCheckBoxElementId' + suffixfText,
                        tagname: "INPUT",
                        style: this._applyStyles.GetCheckboxContainer(),
                        onValueChange: this.attributeSelectionOnChangeHandler.bind(this, attributename),
                        value: this.isAttributeSelected(attributename),
                        accessibilityLabel: fieldRecord.displayname
                    });
                    // Create the Check all elements container
                    var selectAllCheckBoxElementConatiner = this._context.factory.createElement("CONTAINER", {
                        key: "selectAllCheckBoxElementConatiner" + suffixfText,
                        id: "selectAllCheckBoxElementConatiner" + suffixfText,
                        style: this._applyStyles.GetCheckboxContainer()
                    }, selectAllCheckBoxElement);
                    // Create the Display Name sort button element
                    var displayNameLabelElement = this._context.factory.createElement("LABEL", {
                        key: "displayNameLabelElementKey" + suffixfText,
                        id: "displayNameLabelElementId" + suffixfText,
                        style: this._applyStyles.GetCellComponent(),
                        hidden: false,
                        padding: this._context.theming.measures.measure075,
                        onClick: null,
                    }, fieldRecord.displayname + "(" + fieldRecord.datatype + ")");
                    // Create the Display Name sort button element container
                    var displayNameLabelElementConatiner = this._context.factory.createElement("CONTAINER", {
                        key: "displayNameLabelElementConatiner" + suffixfText,
                        id: "displayNameLabelElementConatiner" + suffixfText,
                        style: this._applyStyles.GetCellContainer()
                    }, displayNameLabelElement);
                    attributesGridRowContainerElements.push(displayNameLabelElementConatiner);
                    var rowContainerWithoutCheckbox = this._context.factory.createElement("CONTAINER", {
                        key: "rowContainerWithoutCheckbox" + suffixfText,
                        id: "rowContainerWithoutCheckbox" + suffixfText,
                        style: this._applyStyles.GetElementsContainer()
                    }, attributesGridRowContainerElements);
                    // Create the container for all the header elements
                    attributesGridRowContainer = this._context.factory.createElement("CONTAINER", {
                        key: "attributesGridRowContainer" + suffixfText,
                        id: "attributesGridRowContainer" + suffixfText,
                        style: this._applyStyles.GetRowContainer()
                    }, [selectAllCheckBoxElementConatiner, rowContainerWithoutCheckbox]);
                    return attributesGridRowContainer;
                };
                ExportToExcelControl.prototype.renderGridRowElementsForNormalViewPort = function (fieldRecord) {
                    var attributesGridRowContainer;
                    var attributesGridRowContainerElements = [];
                    var attributename = fieldRecord.name;
                    var suffixfText = this._selectedEntity + attributename;
                    // Create the checkbox element
                    var selectAllCheckBoxElement = this._context.factory.createElement("BOOLEAN", {
                        type: "checkbox",
                        role: "checkbox",
                        tabIndex: 0,
                        id: 'selectAllCheckBoxElementKey' + suffixfText,
                        key: 'selectAllCheckBoxElementId' + suffixfText,
                        tagname: "INPUT",
                        style: this._applyStyles.GetCheckboxContainer(),
                        onValueChange: this.attributeSelectionOnChangeHandler.bind(this, attributename),
                        value: this.isAttributeSelected(attributename),
                        accessibilityLabel: fieldRecord.displayname
                    });
                    // Create the Check all elements container
                    var selectAllCheckBoxElementConatiner = this._context.factory.createElement("CONTAINER", {
                        key: "selectAllCheckBoxElementConatiner" + suffixfText,
                        id: "selectAllCheckBoxElementConatiner" + suffixfText,
                        style: this._applyStyles.GetCheckboxContainer()
                    }, selectAllCheckBoxElement);
                    // Create the Display Name sort button element
                    var displayNameLabelElement = this._context.factory.createElement("LABEL", {
                        key: "displayNameLabelElementKey" + suffixfText,
                        id: "displayNameLabelElementId" + suffixfText,
                        style: this._applyStyles.GetCellComponent(),
                        hidden: false,
                        padding: this._context.theming.measures.measure075,
                        onClick: null,
                    }, fieldRecord.displayname);
                    // Create the Display Name sort button element container
                    var displayNameLabelElementConatiner = this._context.factory.createElement("CONTAINER", {
                        key: "displayNameLabelElementConatiner" + suffixfText,
                        id: "displayNameLabelElementConatiner" + suffixfText,
                        style: this._applyStyles.GetCellContainer()
                    }, displayNameLabelElement);
                    attributesGridRowContainerElements.push(displayNameLabelElementConatiner);
                    // Create the Name sort button element
                    var nameLabelElement = this._context.factory.createElement("LABEL", {
                        key: "nameLabelElementKey" + suffixfText,
                        id: "nameLabelElementId" + suffixfText,
                        style: this._applyStyles.GetCellComponent(),
                        hidden: false,
                        padding: this._context.theming.measures.measure075,
                        onClick: null,
                    }, fieldRecord.name);
                    // Create the Name sort button element container
                    var nameLabelElementConatiner = this._context.factory.createElement("CONTAINER", {
                        key: "nameLabelElementKeyContainer" + suffixfText,
                        id: "nameLabelElementKeyContainer" + suffixfText,
                        style: this._applyStyles.GetCellContainer()
                    }, nameLabelElement);
                    attributesGridRowContainerElements.push(nameLabelElementConatiner);
                    // Create the Type sort button element
                    var typeLabelElement = this._context.factory.createElement("LABEL", {
                        key: "typeLabelElementKey" + suffixfText,
                        id: "typeLabelElementId" + suffixfText,
                        style: this._applyStyles.GetCellComponent(),
                        hidden: false,
                        padding: this._context.theming.measures.measure075,
                        onClick: null,
                    }, fieldRecord.datatype);
                    // Create the Type sort button element container
                    var typeLabelElementConatiner = this._context.factory.createElement("CONTAINER", {
                        key: "typeLabelElementKeyContainer" + suffixfText,
                        id: "typeLabelElementKeyContainer" + suffixfText,
                        style: this._applyStyles.GetCellContainer()
                    }, typeLabelElement);
                    attributesGridRowContainerElements.push(typeLabelElementConatiner);
                    // Create the container for the button elements
                    var rowContainerWithoutCheckbox = this._context.factory.createElement("CONTAINER", {
                        key: "rowContainerWithoutCheckbox" + suffixfText,
                        id: "rowContainerWithoutCheckbox" + suffixfText,
                        style: this._applyStyles.GetElementsContainer()
                    }, attributesGridRowContainerElements);
                    // Create the container for all the header elements
                    attributesGridRowContainer = this._context.factory.createElement("CONTAINER", {
                        key: "attributesGridRowContainer" + suffixfText,
                        id: "attributesGridRowContainer" + suffixfText,
                        style: this._applyStyles.GetRowContainer()
                    }, [selectAllCheckBoxElementConatiner, rowContainerWithoutCheckbox]);
                    return attributesGridRowContainer;
                };
                ExportToExcelControl.prototype.processDefaultAttributeSelected = function () {
                    var xmlDocument = ExportToExcel.XmlUtilities.ParseXml(this._entityFetchXml);
                    var entitiesNodeCollection = $(xmlDocument).find("fetch>entity");
                    var entityNode = entitiesNodeCollection[0];
                    var tempEntityDataObj = this.retrieveEntityDetailsDataObj(entityNode.getAttribute("name"));
                    var attributeNodeCollection = $(xmlDocument).find("fetch>entity>attribute");
                    var index = 0;
                    while (index < attributeNodeCollection.length) {
                        var attributeChildNode = attributeNodeCollection[index];
                        tempEntityDataObj.selectedAttributes.push(attributeChildNode.getAttribute("name"));
                        index++;
                    }
                    this.updateEntityDetailsDataObject(this._relIdOfCurrentEntity, tempEntityDataObj.selectedAttributes);
                    //Process for Link-Entity and It's Attributes
                    var linkEntityNodeCollection = $(xmlDocument).find("fetch>entity>link-entity");
                    index = 0;
                    while (index < linkEntityNodeCollection.length) {
                        var linkEntityNode = linkEntityNodeCollection[index];
                        var linkEntityNodeFromAttribute = linkEntityNode.getAttribute("from");
                        var linkEntityNodeToAttribute = linkEntityNode.getAttribute("to");
                        var linkEntityName = linkEntityNode.getAttribute("name");
                        tempEntityDataObj = this.retrieveEntityDetailsDataObjUsingAttributes(linkEntityNodeFromAttribute, linkEntityNodeToAttribute, linkEntityName);
                        //Iterate all attributes within this Link-entity and Set SelectedAttributes to true
                        var linkEntityNodeAttributeNodeCollection = $(xmlDocument).find("fetch>entity>link-entity>attribute");
                        var attributeNodeIndex = 0;
                        while (attributeNodeIndex < linkEntityNodeAttributeNodeCollection.length) {
                            var attributeNode = linkEntityNodeAttributeNodeCollection[attributeNodeIndex];
                            tempEntityDataObj.selectedAttributes.push(attributeNode.getAttribute("name"));
                            attributeNodeIndex++;
                        }
                        this.updateEntityDetailsDataObject(tempEntityDataObj.relid, tempEntityDataObj.selectedAttributes);
                        index++;
                    }
                };
                ExportToExcelControl.prototype.processEntityXmlAndCreateOptions = function () {
                    var that = this;
                    if (this._entitiesDataObject.length <= 0) {
                        var entitiesXmlData = this.retrieveEntitiesXml().then(function (response) {
                            if (response.EntitiesXml) {
                                var xmlDocument = ExportToExcel.XmlUtilities.ParseXml(CrmEncodeDecode.CrmHtmlDecode(response.EntitiesXml));
                                var count = 0;
                                var entitiesNodeCollection = $(xmlDocument).find("entities>entity");
                                while (count < entitiesNodeCollection.length) {
                                    var entityNode = entitiesNodeCollection[count];
                                    var displayName = entityNode.getAttribute("displayname");
                                    var relid = entityNode.getAttribute("relid");
                                    var alias = entityNode.getAttribute("alias");
                                    var name_1 = entityNode.getAttribute("name");
                                    var otc = entityNode.getAttribute("otc");
                                    var entitydisplayname = entityNode.getAttribute("entitydisplayname");
                                    var griddisplaynamemask = entityNode.getAttribute("griddisplaynamemask");
                                    var toattr = entityNode.getAttribute("toattr");
                                    var fromattr = entityNode.getAttribute("fromattr");
                                    var relationshipname = entityNode.getAttribute("relationshipname");
                                    var relationshipid = entityNode.getAttribute("relationshipid");
                                    var entityDetailsDataObject = new ExportToExcel.EntityDetails();
                                    {
                                        if (ExportToExcel.StringUtilities.IsNullOrEmptyString(alias)) {
                                            that._relIdOfCurrentEntity = relid;
                                            that._selectedEntity = name_1;
                                        }
                                        entityDetailsDataObject.alias = alias, entityDetailsDataObject.displayname = displayName, entityDetailsDataObject.name = name_1, entityDetailsDataObject.entitydisplayname = entitydisplayname, entityDetailsDataObject.griddisplaynamemask = griddisplaynamemask, entityDetailsDataObject.toattr = toattr, entityDetailsDataObject.fromattr = fromattr, entityDetailsDataObject.relid = relid, entityDetailsDataObject.relationshipid = relationshipid, entityDetailsDataObject.relationshipname = relationshipname, otc = !ExportToExcel.StringUtilities.IsNullOrUndefined(otc) ? parseInt(otc) : 0;
                                    }
                                    that._entitiesDataObject.push(entityDetailsDataObject);
                                    that._entitiesSelectorOptions.push(ExportToExcel.ExportControlUtils.CreateOption(displayName, relid, name_1, otc, count));
                                    count++;
                                }
                                that.processFieldAndPropertiesXmlData(that._entityFetchXml);
                            }
                        }, (function (err) {
                            // todo: log failure to retrieve xml
                            // In case of failure. set the mapxml to failedToGetMapXml and rerender
                            var excelControlFailureEvent = new ExportToExcel.ExcelControlFailureEvent(that._entityTypeCode, that._exportType == 2 ? "Dynamic" : "Pivot", err.message);
                            Xrm.Reporting.reportEvent(excelControlFailureEvent);
                            that._context.navigation.openAlertDialog({
                                text: "Failed to Get Entities", confirmButtonLabel: "Error"
                            });
                        }));
                    }
                };
                ExportToExcelControl.prototype.CreateOptionSetControl = function (context, controlKey, controlId, options) {
                    return context.factory.createElement("SELECT", {
                        key: controlKey,
                        id: controlId,
                        //labelledByElementId: this._context.accessibility.getUniqueId("FieldsToRenderLabel"),
                        //describedByElementId: this._context.accessibility.getUniqueId("FieldsToRenderDescription_label"),
                        title: this._context.resources.getString("SystemCustomization.SelectRecordTypeTooltip"),
                        onChange: this.entitySelectorOptionChangeHandler.bind(this),
                        value: this._currentSelectedOption,
                        disabled: false,
                        options: options,
                        style: ExportToExcel.ExportControlUtils.SelectStyle(context)
                    }, null);
                };
                ExportToExcelControl.prototype.entitySelectorOptionChangeHandler = function (option) {
                    this._currentSelectedOption = option;
                    this._selectedEntity = option.EntityName;
                    this._entityTypeCode = option.otc;
                    this._relIdOfCurrentEntity = option.text;
                    var entityFieldsDataObject = this.retrieveEntityFieldsDataObj(this._selectedEntity);
                    if (ExportToExcel.StringUtilities.IsNullOrUndefined(entityFieldsDataObject.entityAttributesList) && entityFieldsDataObject.entityAttributesList.length > 0) {
                        if (entityFieldsDataObject.sortedOn != ExportToExcelControl.EntityFieldsSortType) {
                            entityFieldsDataObject.entityAttributesList = ExportToExcel.ExportControlUtils.sortOnKeysAttributeFile(entityFieldsDataObject.entityAttributesList);
                            this.updateEntityFieldsDataObj(this._selectedEntity, entityFieldsDataObject);
                        }
                        this.renderControl(true);
                    }
                    else {
                        this._isInitialEntityDataFetched = false;
                        this.processFieldAndPropertiesXmlData("");
                    }
                };
                ExportToExcelControl.prototype.attributeSelectionOnChangeHandler = function (attributeName, checked) {
                    checked ? this.addAttributeToFetchXml(attributeName) : this.removeAttributeFromFetchXml(attributeName);
                    this.updateLayoutXml(attributeName, checked);
                    // call the requestrender method to re-render the controls
                    this.renderControl(true);
                };
                ExportToExcelControl.prototype.addAttributeToFetchXml = function (attributeName) {
                    // Update the EntityObject
                    var tempEntityObject = this.retrieveEntityDetailsDataObj(this._relIdOfCurrentEntity);
                    tempEntityObject.selectedAttributes.push(attributeName);
                    this.updateEntityDetailsDataObject(this._relIdOfCurrentEntity, tempEntityObject.selectedAttributes);
                    //Update the updateFetchXml
                    var xmlDocument = ExportToExcel.XmlUtilities.ParseXml(this._updatedFetchXml);
                    var entitiesNodeCollection = $(xmlDocument).find("fetch>entity");
                    var entityNode = entitiesNodeCollection[0];
                    //Validate if it is from Same Primary Entity
                    if (ExportToExcel.StringUtilities.IsNullOrEmpty(tempEntityObject.alias)) {
                        var attributeNode = xmlDocument.createElement("attribute");
                        attributeNode.setAttribute("name", attributeName);
                        entityNode.appendChild(attributeNode);
                        this._updatedFetchXml = (new XMLSerializer()).serializeToString(xmlDocument);
                        console.log(this._updatedFetchXml);
                    }
                    else {
                        //Check if the LinkedEntity Exist then add to them or Create Linked Entity Node
                        var linkEntityNodeCollection = $(xmlDocument).find("fetch>entity>link-entity[name='" + this._selectedEntity + "']");
                        if (linkEntityNodeCollection.length == 1) {
                            var childNode = linkEntityNodeCollection[0];
                            if (childNode.getAttribute("alias") == tempEntityObject.alias) {
                                //Append within this Linked Entity Node
                                var attributeNode = xmlDocument.createElement("attribute");
                                attributeNode.setAttribute("name", attributeName);
                                childNode.appendChild(attributeNode);
                                this._updatedFetchXml = (new XMLSerializer()).serializeToString(xmlDocument);
                                console.log(this._updatedFetchXml);
                            }
                        }
                        else {
                            // No Linked-Entity Node Found, So Create one and Append this attribute
                            var newLinkEntityNode = xmlDocument.createElement("link-entity");
                            newLinkEntityNode.setAttribute("name", tempEntityObject.name);
                            newLinkEntityNode.setAttribute("to", tempEntityObject.toattr);
                            newLinkEntityNode.setAttribute("from", tempEntityObject.fromattr);
                            newLinkEntityNode.setAttribute("link-type", "outer");
                            newLinkEntityNode.setAttribute("alias", tempEntityObject.alias);
                            var attributeNode = xmlDocument.createElement("attribute");
                            attributeNode.setAttribute("name", attributeName);
                            newLinkEntityNode.appendChild(attributeNode);
                            entityNode.appendChild(newLinkEntityNode);
                            this._updatedFetchXml = (new XMLSerializer()).serializeToString(xmlDocument);
                            console.log(this._updatedFetchXml);
                        }
                    }
                    this._notifyOutputChanged();
                };
                ExportToExcelControl.prototype.removeAttributeFromFetchXml = function (attributeName) {
                    var tempEntityObject = this.retrieveEntityDetailsDataObj(this._relIdOfCurrentEntity);
                    var index = tempEntityObject.selectedAttributes.indexOf(attributeName);
                    if (index != -1) {
                        tempEntityObject.selectedAttributes.splice(index, 1);
                    }
                    //this.retrieveEntityDetailsDataObj(this._relIdOfCurrentEntity).selectedAttributes.slice(index, 1);
                    //Update the updateFetchXml
                    var xmlDocument = ExportToExcel.XmlUtilities.ParseXml(this._updatedFetchXml);
                    var entitiesNodeCollection = $(xmlDocument).find("fetch>entity");
                    var entityNode = entitiesNodeCollection[0];
                    //Validate if it is from Same Primary Entity
                    if (ExportToExcel.StringUtilities.IsNullOrEmpty(tempEntityObject.alias)) {
                        var attributeNodeCollection = $(xmlDocument).find("fetch>entity>attribute[name='" + attributeName + "']");
                        if (attributeNodeCollection.length == 1) {
                            entityNode.removeChild(attributeNodeCollection[0]);
                            this._updatedFetchXml = (new XMLSerializer()).serializeToString(xmlDocument);
                            console.log(this._updatedFetchXml);
                        }
                    }
                    else {
                        //Check if the LinkedEntity Exist then Remove from the Node
                        var linkEntityNodeCollection = $(xmlDocument).find("fetch>entity>link-entity[name='" + this._selectedEntity + "']");
                        var linkEntityNodeAtrributeNodeCollection = $(xmlDocument).find("fetch>entity>link-entity[name='" + this._selectedEntity + "']>attribute[name='" + attributeName + "']");
                        if (linkEntityNodeAtrributeNodeCollection.length == 1) {
                            var childNode = linkEntityNodeAtrributeNodeCollection[0];
                            linkEntityNodeCollection[0].removeChild(childNode);
                        }
                        //Check if Attribute in this linked entity Node is Null
                        var linkEntityattributeNodeCollection = $(xmlDocument).find("fetch>entity>link-entity[name='" + this._selectedEntity + "']>attribute");
                        if (linkEntityattributeNodeCollection.length == 0) {
                            entityNode.removeChild(linkEntityNodeCollection[0]);
                        }
                        this._updatedFetchXml = (new XMLSerializer()).serializeToString(xmlDocument);
                        console.log(this._updatedFetchXml);
                    }
                    this._notifyOutputChanged();
                };
                ExportToExcelControl.prototype.updateLayoutXml = function (attributeName, isAttributeAdded) {
                    var tempEntityObject = this.retrieveEntityDetailsDataObj(this._relIdOfCurrentEntity);
                    var xmlDocument = ExportToExcel.XmlUtilities.ParseXml(this._entityLayoutXml);
                    var rowNodeCollection = $(xmlDocument).find("grid>row");
                    var rowNode = rowNodeCollection[0];
                    if (isAttributeAdded) {
                        //Validate if it is from Same Primary Entity
                        if (ExportToExcel.StringUtilities.IsNullOrEmpty(tempEntityObject.alias)) {
                            var newCellNode = xmlDocument.createElement("cell");
                            newCellNode.setAttribute("name", attributeName);
                            rowNode.appendChild(newCellNode);
                            this._entityLayoutXml = (new XMLSerializer()).serializeToString(xmlDocument);
                            console.log(this._entityLayoutXml);
                        }
                        else {
                            //Check if the LinkedEntity Exist then add to them or Create Linked Entity Node
                            var mappedColumnName = tempEntityObject.alias + "." + attributeName;
                            var linkEntityCellNodeCollection = $(xmlDocument).find("grid>row>cell[name='" + tempEntityObject.alias + "." + attributeName + "']");
                            if (linkEntityCellNodeCollection.length == 1) {
                                var childNode = linkEntityCellNodeCollection[0];
                                if (childNode.getAttribute("name") == mappedColumnName) {
                                    //Append within this cell node
                                    if (ExportToExcel.StringUtilities.IsNullOrUndefined(childNode.getAttribute("relationshipid"))) {
                                        var relationshipidNode = xmlDocument.createElement("attribute");
                                        childNode.setAttribute("relationshipid", tempEntityObject.relationshipid);
                                        childNode.setAttribute("relationshipname", tempEntityObject.relationshipname);
                                        childNode.appendChild(relationshipidNode);
                                        this._entityLayoutXml = (new XMLSerializer()).serializeToString(xmlDocument);
                                        console.log(this._entityLayoutXml);
                                    }
                                }
                            }
                            else {
                                // No Linked-Entity Node Found, So Create one and Append this attribute
                                var newCellNode = xmlDocument.createElement("cell");
                                newCellNode.setAttribute("name", mappedColumnName);
                                newCellNode.setAttribute("relatedentityname", tempEntityObject.name);
                                newCellNode.setAttribute("relatedentityattr", tempEntityObject.fromattr);
                                newCellNode.setAttribute("primaryentityattr", tempEntityObject.toattr);
                                newCellNode.setAttribute("relationshipid", tempEntityObject.relationshipid);
                                newCellNode.setAttribute("relationshipname", tempEntityObject.relationshipname);
                                rowNode.appendChild(newCellNode);
                                this._entityLayoutXml = (new XMLSerializer()).serializeToString(xmlDocument);
                                console.log(this._entityLayoutXml);
                            }
                        }
                    }
                    else {
                        //Validate if it is from Same Primary Entity
                        if (ExportToExcel.StringUtilities.IsNullOrEmpty(tempEntityObject.alias)) {
                            var cellNodeCollection = $(xmlDocument).find("grid>row>cell[name='" + attributeName + "']");
                            if (cellNodeCollection.length == 1) {
                                rowNode.removeChild(cellNodeCollection[0]);
                                this._entityLayoutXml = (new XMLSerializer()).serializeToString(xmlDocument);
                                console.log(this._entityLayoutXml);
                            }
                        }
                        else {
                            var mappedColumnName = tempEntityObject.alias + "." + attributeName;
                            //Check if the LinkedEntity Exist then Remove from the Node
                            var cellNodeCollection = $(xmlDocument).find("grid>row>cell[name='" + mappedColumnName + "']");
                            if (cellNodeCollection.length == 1) {
                                rowNode.removeChild(cellNodeCollection[0]);
                            }
                            this._entityLayoutXml = (new XMLSerializer()).serializeToString(xmlDocument);
                            console.log(this._entityLayoutXml);
                        }
                    }
                    this._notifyOutputChanged();
                };
                ExportToExcelControl.prototype.displayNameSortButtonClickHandler = function (control) {
                    var isDescending;
                    if (ExportToExcelControl.EntityFieldsSortType === ExportToExcel.EntityFieldsSortByField.DisplayName) {
                        isDescending = (this.sortType === 'asc') ? false : true;
                        this.sortType = (this.sortType === 'asc') ? 'desc' : 'asc';
                    }
                    else {
                        this.sortType = 'asc';
                        isDescending = true;
                    }
                    ExportToExcelControl.EntityFieldsSortType = ExportToExcel.EntityFieldsSortByField.DisplayName;
                    var entityFieldsDataObject = this.retrieveEntityFieldsDataObj(this._selectedEntity);
                    var sortedList = ExportToExcel.ExportControlUtils.sortOnKeysAttributeFile(entityFieldsDataObject.entityAttributesList);
                    entityFieldsDataObject.entityAttributesList = (isDescending) ? sortedList : sortedList.reverse();
                    this.updateEntityFieldsDataObj(this._selectedEntity, entityFieldsDataObject);
                    this.renderControl(true);
                };
                ExportToExcelControl.prototype.nameSortButtonClickHandler = function (control) {
                    var isDescending;
                    if (ExportToExcelControl.EntityFieldsSortType === ExportToExcel.EntityFieldsSortByField.Name) {
                        isDescending = (this.sortType === 'asc') ? false : true;
                        this.sortType = (this.sortType === 'asc') ? 'desc' : 'asc';
                    }
                    else {
                        this.sortType = 'asc';
                        isDescending = true;
                    }
                    ExportToExcelControl.EntityFieldsSortType = ExportToExcel.EntityFieldsSortByField.Name;
                    var entityFieldsDataObject = this.retrieveEntityFieldsDataObj(this._selectedEntity);
                    var sortedList = ExportToExcel.ExportControlUtils.sortOnKeysAttributeFile(entityFieldsDataObject.entityAttributesList);
                    entityFieldsDataObject.entityAttributesList = (isDescending) ? sortedList : sortedList.reverse();
                    this.updateEntityFieldsDataObj(this._selectedEntity, entityFieldsDataObject);
                    this.renderControl(true);
                };
                ExportToExcelControl.prototype.dataTypeSortButtonClickHandler = function (control) {
                    var isDescending;
                    if (ExportToExcelControl.EntityFieldsSortType === ExportToExcel.EntityFieldsSortByField.DataType) {
                        isDescending = (this.sortType === 'asc') ? false : true;
                        this.sortType = (this.sortType === 'asc') ? 'desc' : 'asc';
                    }
                    else {
                        this.sortType = 'asc';
                        isDescending = true;
                    }
                    ExportToExcelControl.EntityFieldsSortType = ExportToExcel.EntityFieldsSortByField.DataType;
                    var entityFieldsDataObject = this.retrieveEntityFieldsDataObj(this._selectedEntity);
                    var sortedList = ExportToExcel.ExportControlUtils.sortOnKeysAttributeFile(entityFieldsDataObject.entityAttributesList);
                    entityFieldsDataObject.entityAttributesList = (isDescending) ? sortedList : sortedList.reverse();
                    this.updateEntityFieldsDataObj(this._selectedEntity, entityFieldsDataObject);
                    this.renderControl(true);
                };
                ExportToExcelControl.prototype.processFieldAndPropertiesXmlData = function (fetchXml) {
                    var that = this;
                    var tempFetchXml = fetchXml;
                    var fieldAndPropertiesdata = this.retrieveFeildAndPropertiesXml(fetchXml).then(function (data) {
                        if (data.FieldAndPropertiesXml) {
                            console.log(data);
                            var fieldAndPropertiesXml = ExportToExcel.XmlUtilities.ParseXml(CrmEncodeDecode.CrmHtmlDecode(data.FieldAndPropertiesXml));
                            fieldAndPropertiesdata = "";
                            var count = 0;
                            var propertiesXml = $(fieldAndPropertiesXml).find("fieldpropertiesxml>propertiesxml");
                            var fieldXml = $(fieldAndPropertiesXml).find("fieldpropertiesxml>fieldxml");
                            fieldAndPropertiesXml = null;
                            that.processPropertiesAndFieldXml(propertiesXml, fieldXml);
                            if (!that._processOnlyForInitialFetch) {
                                that.processDefaultAttributeSelected();
                                that._processOnlyForInitialFetch = true;
                            }
                            that._isInitialEntityDataFetched = true;
                            that.renderControl(true);
                        }
                    }, function (err) {
                        var excelControlFailureEvent = new ExportToExcel.ExcelControlFailureEvent(that._entityTypeCode, that._exportType == 2 ? "Dynamic" : "Pivot", err.message);
                        Xrm.Reporting.reportEvent(excelControlFailureEvent);
                        throw err.error();
                    });
                    ;
                };
                ExportToExcelControl.prototype.processPropertiesAndFieldXml = function (propertiesXml, fieldXml) {
                    var entitiesNodeCollectionFromPropertiesXml = $(propertiesXml).find("propertiesxml>entities>entity");
                    var count = 0;
                    while (count < entitiesNodeCollectionFromPropertiesXml.length) {
                        var entityNode = entitiesNodeCollectionFromPropertiesXml[count];
                        var entityName = entityNode.getAttribute('name');
                        var entityFieldsDataObject = this.retrieveEntityFieldsDataObj(entityName);
                        if (entityFieldsDataObject.entityAttributesList.length <= 0) {
                            var fieldsNodeCollectionFromFieldsXml = $(fieldXml).find("fieldXml>entities>entity[name='" + entityName + "']>fields");
                            var fieldsNodeCollectionFromPropertiesXml = $(entityNode).find("entity>fields>field");
                            var fieldsInPropertiesXmlCount = 0;
                            while (fieldsInPropertiesXmlCount < fieldsNodeCollectionFromPropertiesXml.length) {
                                var fieldNodeFromPropertiesXml = fieldsNodeCollectionFromPropertiesXml[fieldsInPropertiesXmlCount];
                                var fieldName = fieldNodeFromPropertiesXml.getAttribute('name');
                                var orgRequired = fieldNodeFromPropertiesXml.getAttribute('requiredlevel');
                                if (!ExportToExcel.StringUtilities.IsNullOrWhiteSpace(fieldName)) {
                                    var fieldNodeFromFieldsXml = fieldsNodeCollectionFromFieldsXml.find("fields>field[name='" + fieldName + "']")[0];
                                    if (!ExportToExcel.StringUtilities.IsNullOrUndefined(fieldNodeFromFieldsXml)) {
                                        var isValidForAdvanceFind = fieldNodeFromFieldsXml.getAttribute("validforadvancedfind") == "true";
                                        var isValidForGrid = fieldNodeFromFieldsXml.getAttribute("validforgrid") == "true";
                                        var fieldDataType = fieldNodeFromFieldsXml.getAttribute("datatype");
                                        // add conditions and get the input parameter for dynamic excel
                                        if (isValidForGrid) {
                                            var displayNameText = $(fieldNodeFromPropertiesXml).find("displaynames>displayname[languagecode=" + this._context.client.languageCode + "]")[0].getAttribute('description');
                                            if (!ExportToExcel.StringUtilities.IsNullOrWhiteSpace(displayNameText)) {
                                                var newFieldRecord = new ExportToExcel.FieldRecord();
                                                {
                                                    newFieldRecord.datatype = this.getFriendlyTypeNames(fieldDataType), newFieldRecord.name = fieldName, newFieldRecord.displayname = displayNameText, newFieldRecord.orgrequired = orgRequired, newFieldRecord.required = false;
                                                }
                                                entityFieldsDataObject.entityAttributesList.push(newFieldRecord);
                                            }
                                        }
                                    }
                                }
                                fieldsInPropertiesXmlCount++;
                            }
                        }
                        // Sort for the Current Entity
                        entityFieldsDataObject.entityAttributesList = ExportToExcel.ExportControlUtils.sortOnKeysAttributeFile(entityFieldsDataObject.entityAttributesList);
                        this.updateEntityFieldsDataObj(entityName, entityFieldsDataObject);
                        count++;
                    }
                };
                ExportToExcelControl.prototype.retrieveEntityFieldsDataObj = function (entityName) {
                    var entityFiledsDataObject = new ExportToExcel.EntityFields();
                    var entityFieldsRecordExists = false;
                    for (var i = 0; i < this._entitiesFieldsDataObj.length; i++) {
                        if (this._entitiesFieldsDataObj[i].name == entityName) {
                            entityFiledsDataObject = this._entitiesFieldsDataObj[i];
                            entityFieldsRecordExists = true;
                            break;
                        }
                    }
                    if (!entityFieldsRecordExists) {
                        entityFiledsDataObject.name = entityName;
                        entityFiledsDataObject.entityAttributesList = [];
                        this._entitiesFieldsDataObj.push(entityFiledsDataObject);
                    }
                    return entityFiledsDataObject;
                };
                ExportToExcelControl.prototype.retrieveEntityDetailsDataObj = function (relationshipId) {
                    var entityDetailsDataObject = new ExportToExcel.EntityDetails();
                    for (var i = 0; i < this._entitiesDataObject.length; i++) {
                        if (this._entitiesDataObject[i].relid == relationshipId) {
                            entityDetailsDataObject = this._entitiesDataObject[i];
                            break;
                        }
                    }
                    return entityDetailsDataObject;
                };
                ExportToExcelControl.prototype.retrieveEntityDetailsDataObjUsingAttributes = function (fromAttribute, toAttribute, entityName) {
                    var entityDetailsDataObject = new ExportToExcel.EntityDetails();
                    for (var i = 0; i < this._entitiesDataObject.length; i++) {
                        if (this._entitiesDataObject[i].fromattr == fromAttribute && this._entitiesDataObject[i].toattr == toAttribute && this._entitiesDataObject[i].name == entityName) {
                            entityDetailsDataObject = this._entitiesDataObject[i];
                            break;
                        }
                    }
                    return entityDetailsDataObject;
                };
                ExportToExcelControl.prototype.updateEntityDetailsDataObject = function (relationshipId, selectedAttributes) {
                    for (var i = 0; i < this._entitiesDataObject.length; i++) {
                        if (this._entitiesDataObject[i].relid == relationshipId) {
                            this._entitiesDataObject[i].selectedAttributes = selectedAttributes;
                            break;
                        }
                    }
                };
                ExportToExcelControl.prototype.updateEntityFieldsDataObj = function (entityName, entityFiledsDataObject) {
                    for (var i = 0; i < this._entitiesFieldsDataObj.length; i++) {
                        if (this._entitiesFieldsDataObj[i].name == entityName) {
                            entityFiledsDataObject.sortedOn = ExportToExcelControl.EntityFieldsSortType;
                            this._entitiesFieldsDataObj[i] = entityFiledsDataObject;
                            break;
                        }
                    }
                };
                ExportToExcelControl.prototype.isAttributeSelected = function (attributeName) {
                    var matchingRecords = this.retrieveEntityDetailsDataObj(this._relIdOfCurrentEntity).selectedAttributes.filter(function (selectedAttributesObject) { return selectedAttributesObject == attributeName; });
                    return matchingRecords.length > 0;
                };
                ExportToExcelControl.prototype.renderControl = function (render, callback) {
                    if (render === void 0) { render = false; }
                    if (callback === void 0) { callback = null; }
                    if (render) {
                        this._context.utils.requestRender(callback);
                    }
                };
                ExportToExcelControl.prototype.getFriendlyTypeNames = function (typeName) {
                    if (this._typeNameDictionary[typeName]) {
                        return this._typeNameDictionary[typeName];
                    }
                    else {
                        this._typeNameDictionary[typeName] = ExportToExcel.ExportControlUtils.GetFriendlyTypeNames(typeName, this._context);
                        return this._typeNameDictionary[typeName];
                    }
                };
                ExportToExcelControl.prototype.getViewPortWidth = function () {
                    if (document.compatMode == "CSS1Compat")
                        return {
                            w: document.documentElement.clientWidth,
                            h: document.documentElement.clientHeight
                        };
                    // For browsers in Quirks mode
                    return { w: document.body.clientWidth, h: document.body.clientHeight };
                };
                return ExportToExcelControl;
            }());
            ExportToExcel.ExportToExcelControl = ExportToExcelControl;
        })(ExportToExcel = AppCommon.ExportToExcel || (AppCommon.ExportToExcel = {}));
    })(AppCommon = MscrmControls.AppCommon || (MscrmControls.AppCommon = {}));
})(MscrmControls || (MscrmControls = {}));
/**
* @license Copyright (c) Microsoft Corporation. All rights reserved.
*/
// Constansts
/// <reference path="Constants/ExportToExcelConstants.ts" />
/// <reference path="inputsoutputs.g.ts" />
/// <reference path="ExportToExcelControl.ts" /> 
var ODataContract;
(function (ODataContract) {
    var GetEntitiesXmlRequest = (function () {
        function GetEntitiesXmlRequest(entityTypeCode, selectedEntities) {
            this.EntityTypeCode = entityTypeCode;
            this.FetchXml = selectedEntities;
        }
        GetEntitiesXmlRequest.prototype.getMetadata = function () {
            var metadata = {
                boundParameter: null,
                parameterTypes: {
                    "EntityTypeCode": {
                        "typeName": "Edm.Int32",
                        "structuralProperty": 1,
                    },
                    "FetchXml": {
                        "typeName": "Edm.String",
                        "structuralProperty": 1,
                    },
                },
                operationName: "GetEntitiesXml",
                operationType: 0,
            };
            return metadata;
        };
        return GetEntitiesXmlRequest;
    }());
    ODataContract.GetEntitiesXmlRequest = GetEntitiesXmlRequest;
})(ODataContract || (ODataContract = {}));
var ODataContract;
(function (ODataContract) {
    var GetFieldAndPropertiesXmlRequest = (function () {
        function GetFieldAndPropertiesXmlRequest(entityTypeCode, selectedEntities) {
            this.EntityTypeCode = entityTypeCode;
            this.FetchXml = selectedEntities;
        }
        GetFieldAndPropertiesXmlRequest.prototype.getMetadata = function () {
            var metadata = {
                boundParameter: null,
                parameterTypes: {
                    "EntityTypeCode": {
                        "typeName": "Edm.Int32",
                        "structuralProperty": 1,
                    },
                    "FetchXml": {
                        "typeName": "Edm.String",
                        "structuralProperty": 1,
                    },
                },
                operationName: "GetFieldAndPropertiesXml",
                operationType: 0,
            };
            return metadata;
        };
        return GetFieldAndPropertiesXmlRequest;
    }());
    ODataContract.GetFieldAndPropertiesXmlRequest = GetFieldAndPropertiesXmlRequest;
})(ODataContract || (ODataContract = {}));
var MscrmControls;
(function (MscrmControls) {
    var AppCommon;
    (function (AppCommon) {
        var ExportToExcel;
        (function (ExportToExcel) {
            'use strict';
            var ExportToExcelStyles = (function () {
                function ExportToExcelStyles(context) {
                    this._contentContainerChild = {};
                    this._getHeaderCheckbox = {};
                    this._getHeaderContainer = {};
                    this._arrowStyles = {};
                    this._arrowStylesNone = {};
                    this._getElementsContainer = {};
                    this._getCheckboxContainer = {};
                    this._getHeaderCheckboxContainer = {};
                    this._getHeaderRowElementsContainer = {};
                    this._getRowCheckbox = {};
                    this._getRowContainer = {};
                    this._getGridContainer = {};
                    this._getCellComponent = {};
                    this._getCellContainer = {};
                    this._getHeaderCellContainer = {};
                    this._getDisplayNameHeaderCellComponent = {};
                    this._getNameHeaderCellComponent = {};
                    this._getTypeHeaderCellComponent = {};
                    this._context = context;
                    this._contentContainerChild = null;
                    this._getHeaderCheckbox = null;
                    this._getHeaderContainer = null;
                    this._arrowStyles = null;
                    this._arrowStylesNone = null;
                    this._getHeaderCheckboxContainer = null;
                    this._getElementsContainer = null;
                    this._getHeaderRowElementsContainer = null;
                    this._getCheckboxContainer = null;
                    this._getRowCheckbox = null;
                    this._getRowContainer = null;
                    this._getGridContainer = null;
                    this._getCellComponent = null;
                    this._getCellContainer = null;
                    this._getDisplayNameHeaderCellComponent = null;
                    this._getNameHeaderCellComponent = null;
                    this._getTypeHeaderCellComponent = null;
                    this._getHeaderCellContainer = null;
                }
                ExportToExcelStyles.prototype.ContentContainerChild = function () {
                    if (this._context.utils.isNullOrUndefined(this._contentContainerChild)) {
                        this._contentContainerChild = {};
                        this._contentContainerChild["display"] = "flex";
                        this._contentContainerChild["flexDirection"] = "column";
                        this._contentContainerChild["border"] = this._context.theming.borders.border02;
                        this._contentContainerChild["white-space"] = "nowrap";
                        this._contentContainerChild["overflow"] = "hidden";
                        this._contentContainerChild["text-overflow"] = "ellipsis";
                    }
                    return this._contentContainerChild;
                };
                ExportToExcelStyles.prototype.GetHeaderCheckbox = function () {
                    if (this._context.utils.isNullOrUndefined(this._getHeaderCheckbox)) {
                        this._getHeaderCheckbox = {};
                        this._getHeaderCheckbox["width"] = "2rem";
                    }
                    return this._getHeaderCheckbox;
                };
                ExportToExcelStyles.prototype.GetHeaderCheckboxContainer = function () {
                    if (this._context.utils.isNullOrUndefined(this._getHeaderCheckboxContainer)) {
                        this._getHeaderCheckboxContainer = {};
                        this._getHeaderCheckboxContainer["marginLeft"] = "5px";
                        this._getHeaderCheckboxContainer["marginRight"] = "5px";
                        this._getHeaderCheckboxContainer["display"] = "none";
                    }
                    return this._getHeaderCheckboxContainer;
                };
                ExportToExcelStyles.prototype.GetCheckboxContainer = function () {
                    if (this._context.utils.isNullOrUndefined(this._getCheckboxContainer)) {
                        this._getCheckboxContainer = {};
                        this._getCheckboxContainer["marginLeft"] = "5px";
                        this._getCheckboxContainer["marginRight"] = "5px";
                        this._getCheckboxContainer[":focus"] = {
                            "outline-offset": "0px !important"
                        };
                    }
                    return this._getCheckboxContainer;
                };
                ExportToExcelStyles.prototype.GetHeaderContainer = function () {
                    if (this._context.utils.isNullOrUndefined(this._getHeaderContainer)) {
                        this._getHeaderContainer = {};
                        this._getHeaderContainer["display"] = "flex";
                        this._getHeaderContainer["borderBottom"] = this._context.theming.borders.border02;
                        this._getHeaderContainer["width"] = "100%";
                        this._getHeaderContainer["flexDirection"] = "row";
                        this._getHeaderContainer["alignItems"] = "center";
                        this._getHeaderContainer["padding"] = ".5rem";
                        this._getHeaderContainer["backgroundColor"] = "lightgrey";
                    }
                    return this._getHeaderContainer;
                };
                ExportToExcelStyles.prototype.GetArrowStyles = function () {
                    if (this._context.utils.isNullOrUndefined(this._arrowStyles)) {
                        this._arrowStyles = {};
                        this._arrowStyles["fontSize"] = "0.75rem";
                        this._arrowStyles["paddingTop"] = "5px";
                        this._arrowStyles["color"] = "#000";
                    }
                    return this._arrowStyles;
                };
                ExportToExcelStyles.prototype.GetArrowStylesNone = function () {
                    if (this._context.utils.isNullOrUndefined(this._arrowStylesNone)) {
                        this._arrowStylesNone = {};
                        this._arrowStylesNone["display"] = "none";
                        this._arrowStylesNone["fontSize"] = "0.75rem";
                        this._arrowStylesNone["paddingTop"] = "5px";
                        this._arrowStylesNone["color"] = "#000";
                    }
                    return this._arrowStylesNone;
                };
                ExportToExcelStyles.prototype.GetElementsContainer = function () {
                    if (this._context.utils.isNullOrUndefined(this._getElementsContainer)) {
                        this._getElementsContainer = {};
                        this._getElementsContainer["display"] = "flex";
                        this._getElementsContainer["flex"] = "1";
                        this._getElementsContainer["flexDirection"] = "row";
                        this._getElementsContainer["alignItems"] = "center";
                        this._getElementsContainer["paddingLeft"] = ".5rem";
                        this._getElementsContainer["paddingRight"] = ".5rem";
                        this._getElementsContainer["width"] = "100%";
                    }
                    return this._getElementsContainer;
                };
                ExportToExcelStyles.prototype.GetHeaderRowElementsContainer = function () {
                    if (this._context.utils.isNullOrUndefined(this._getHeaderRowElementsContainer)) {
                        this._getHeaderRowElementsContainer = {};
                        this._getHeaderRowElementsContainer["display"] = "flex";
                        this._getHeaderRowElementsContainer["flex"] = "1";
                        this._getHeaderRowElementsContainer["flexDirection"] = "row";
                        this._getHeaderRowElementsContainer["alignItems"] = "center";
                        this._getHeaderRowElementsContainer["paddingLeft"] = ".5rem";
                        this._getHeaderRowElementsContainer["paddingRight"] = ".5rem";
                        this._getHeaderRowElementsContainer["width"] = "100%";
                    }
                    return this._getHeaderRowElementsContainer;
                };
                ExportToExcelStyles.prototype.GetRowCheckbox = function () {
                    if (this._context.utils.isNullOrUndefined(this._getRowCheckbox)) {
                        this._getRowCheckbox = {};
                    }
                    return this._getRowCheckbox;
                };
                ExportToExcelStyles.prototype.GetRowContainer = function () {
                    if (this._context.utils.isNullOrUndefined(this._getRowContainer)) {
                        this._getRowContainer = {};
                        this._getRowContainer["display"] = "flex";
                        this._getRowContainer["flexDirection"] = "row";
                        this._getRowContainer["alignItems"] = "center";
                        this._getRowContainer["padding"] = ".25rem";
                    }
                    return this._getRowContainer;
                };
                ExportToExcelStyles.prototype.GetGridContainer = function (height) {
                    this._getGridContainer = {};
                    this._getGridContainer["display"] = "flex";
                    this._getGridContainer["alignItems"] = "flex-start";
                    this._getGridContainer["flexDirection"] = "column";
                    this._getGridContainer["boxShadow"] = "0 0 0 1px #cccccc inset";
                    this._getGridContainer["overflow"] = "auto";
                    this._getGridContainer["height"] = height;
                    return this._getGridContainer;
                };
                ExportToExcelStyles.prototype.GetCellComponent = function () {
                    if (this._context.utils.isNullOrUndefined(this._getCellComponent)) {
                        this._getCellComponent = {};
                        this._getCellComponent["height"] = "100%";
                        this._getCellComponent["cursor"] = "pointer";
                        this._getCellComponent["textOverflow"] = "ellipsis";
                        this._getCellComponent["whiteSpace"] = "nowrap";
                        this._getCellComponent["overflow"] = "hidden";
                    }
                    return this._getCellComponent;
                };
                ExportToExcelStyles.prototype.GetCellContainer = function () {
                    if (this._context.utils.isNullOrUndefined(this._getCellContainer)) {
                        this._getCellContainer = {};
                        this._getCellContainer["display"] = "flex";
                        this._getCellContainer["flex"] = "0 0 15.8rem";
                        this._getCellContainer["textOverflow"] = "ellipsis";
                        this._getCellContainer["whiteSpace"] = "nowrap";
                        this._getCellContainer["height"] = "100%";
                        this._getCellContainer["overflow"] = "hidden";
                    }
                    return this._getCellContainer;
                };
                ExportToExcelStyles.prototype.GetDisplayNameHeaderCellComponent = function () {
                    if (this._context.utils.isNullOrUndefined(this._getDisplayNameHeaderCellComponent)) {
                        this._getDisplayNameHeaderCellComponent = {};
                        this._getDisplayNameHeaderCellComponent["height"] = "100%";
                        this._getDisplayNameHeaderCellComponent["cursor"] = "pointer";
                        this._getDisplayNameHeaderCellComponent["textOverflow"] = "ellipsis";
                        this._getDisplayNameHeaderCellComponent["whiteSpace"] = "nowrap";
                        this._getDisplayNameHeaderCellComponent["border"] = "0px";
                        this._getDisplayNameHeaderCellComponent["backgroundColor"] = "transparent";
                        this._getDisplayNameHeaderCellComponent["overflow"] = "hidden";
                        this._getDisplayNameHeaderCellComponent[":focus"] = {
                            "border": "1px dotted"
                        };
                        this._getDisplayNameHeaderCellComponent["padding"] = "0px";
                    }
                    return this._getDisplayNameHeaderCellComponent;
                };
                ExportToExcelStyles.prototype.GetNameHeaderCellComponent = function () {
                    if (this._context.utils.isNullOrUndefined(this._getNameHeaderCellComponent)) {
                        this._getNameHeaderCellComponent = {};
                        this._getNameHeaderCellComponent["height"] = "100%";
                        this._getNameHeaderCellComponent["cursor"] = "pointer";
                        this._getNameHeaderCellComponent["textOverflow"] = "ellipsis";
                        this._getNameHeaderCellComponent["whiteSpace"] = "nowrap";
                        this._getNameHeaderCellComponent["border"] = "0px";
                        this._getNameHeaderCellComponent["backgroundColor"] = "transparent";
                        this._getNameHeaderCellComponent["overflow"] = "hidden";
                        this._getNameHeaderCellComponent["padding"] = "0px";
                        this._getNameHeaderCellComponent[":focus"] = {
                            "border": "1px dotted"
                        };
                    }
                    return this._getNameHeaderCellComponent;
                };
                ExportToExcelStyles.prototype.GetTypeHeaderCellComponent = function () {
                    if (this._context.utils.isNullOrUndefined(this._getTypeHeaderCellComponent)) {
                        this._getTypeHeaderCellComponent = {};
                        this._getTypeHeaderCellComponent["height"] = "100%";
                        this._getTypeHeaderCellComponent["cursor"] = "pointer";
                        this._getTypeHeaderCellComponent["textOverflow"] = "ellipsis";
                        this._getTypeHeaderCellComponent["whiteSpace"] = "nowrap";
                        this._getTypeHeaderCellComponent["border"] = "0px";
                        this._getTypeHeaderCellComponent["backgroundColor"] = "transparent";
                        this._getTypeHeaderCellComponent["overflow"] = "hidden";
                        this._getTypeHeaderCellComponent["padding"] = "0px";
                        this._getTypeHeaderCellComponent[":focus"] = {
                            "border": "1px dotted"
                        };
                    }
                    return this._getTypeHeaderCellComponent;
                };
                ExportToExcelStyles.prototype.GetHeaderCellContainer = function () {
                    if (this._context.utils.isNullOrUndefined(this._getHeaderCellContainer)) {
                        this._getHeaderCellContainer = {};
                        this._getHeaderCellContainer["display"] = "flex";
                        this._getHeaderCellContainer["flex"] = "0 0 15.8rem";
                        this._getHeaderCellContainer["textOverflow"] = "ellipsis";
                        this._getHeaderCellContainer["whiteSpace"] = "nowrap";
                        this._getHeaderCellContainer["overflow"] = "hidden";
                        this._getHeaderCellContainer["height"] = "100%";
                    }
                    return this._getHeaderCellContainer;
                };
                return ExportToExcelStyles;
            }());
            ExportToExcel.ExportToExcelStyles = ExportToExcelStyles;
            var Sytle = (function () {
                function Sytle() {
                    this.main_container = {
                        "font-family": "'Segoe UI Regular', 'SegoeUI', 'Segoe UI'",
                        "display": "flex",
                        "flexDirection": "column"
                    };
                    this.entity_label = {};
                    this.entity_combo = {};
                }
                return Sytle;
            }());
            ExportToExcel.Sytle = Sytle;
        })(ExportToExcel = AppCommon.ExportToExcel || (AppCommon.ExportToExcel = {}));
    })(AppCommon = MscrmControls.AppCommon || (MscrmControls.AppCommon = {}));
})(MscrmControls || (MscrmControls = {}));
/**
* @license Copyright (c) Microsoft Corporation.  All rights reserved.
*/
/// <reference path="../privatereferences.ts"/>
var MscrmControls;
(function (MscrmControls) {
    var AppCommon;
    (function (AppCommon) {
        var ExportToExcel;
        (function (ExportToExcel) {
            'use strict';
            var ExportControlUtils = (function () {
                function ExportControlUtils() {
                }
                ExportControlUtils.CreateOption = function (label, text, entityName, otc, value) {
                    return { text: text, Label: label, Value: value, EntityName: entityName, otc: otc, Color: "" };
                };
                ExportControlUtils.SelectStyle = function (context) {
                    var _a = context.theming, noneborderstyle = _a.noneborderstyle, textbox = _a.textbox;
                    var backgroundColor = textbox.backgroundcolor;
                    var color = textbox.contentcolor;
                    var selectStyle = {
                        appearance: "normal",
                        width: "54%",
                        height: "2.5rem",
                        paddingLeft: textbox.horizontalpadding,
                        paddingRight: textbox.horizontalpadding,
                        fontSize: textbox.fontsize,
                        fontWeight: textbox.fontweight,
                        boxShadow: "0 0 0 1px #cccccc inset",
                        backgroundColor: context.theming.colors.whitebackground,
                        color: color
                    };
                    var optionStyle = {
                        backgroundColor: backgroundColor,
                        color: color
                    };
                    return {
                        selectStyle: selectStyle,
                        optionStyle: optionStyle
                    };
                };
                ExportControlUtils.GetExportEntitiesXml = function (EntityTypeCode, FetchXml, context) {
                    var res1;
                    var response;
                    var entitytypecode = EntityTypeCode;
                    var res;
                    var boundParameter = null;
                    var r = new ODataContract.GetEntitiesXmlRequest(entitytypecode, FetchXml);
                    return res1 = context.webAPI.execute(r)
                        .then(function (response) {
                        if (response.ok) {
                            return response.json();
                        } // this will reject the returned promise
                    }, function (err) {
                        throw response.error();
                    });
                };
                ExportControlUtils.GetFieldAndPropertiesXml = function (EntityTypeCode, FetchXml, context) {
                    var res1;
                    var response;
                    var entitytypecode = EntityTypeCode;
                    var res;
                    var boundParameter = null;
                    var r = new ODataContract.GetFieldAndPropertiesXmlRequest(entitytypecode, FetchXml);
                    return res1 = context.webAPI.execute(r)
                        .then(function (response) {
                        if (response.ok) {
                            return response.json();
                        } // this will reject the returned promise
                    }, function (err) {
                        throw response.error();
                    });
                };
                ExportControlUtils.GetFriendlyTypeNames = function (typeName, context) {
                    //Fetch from Resx string and Return the data
                    return context.resources.getString("SystemCustomization.type." + typeName + ".label");
                };
                ExportControlUtils.compare = function (a, b) {
                    var labelA = "";
                    var labelB = "";
                    var sortType = ExportToExcel.ExportToExcelControl.EntityFieldsSortType;
                    switch (sortType) {
                        case EntityFieldsSortByField.DisplayName:
                            labelA = a.displayname.toUpperCase();
                            labelB = b.displayname.toUpperCase();
                            break;
                        case EntityFieldsSortByField.DataType:
                            labelA = a.datatype.toUpperCase();
                            labelB = b.datatype.toUpperCase();
                            break;
                        case EntityFieldsSortByField.Name:
                            labelA = a.name.toUpperCase();
                            labelB = b.name.toUpperCase();
                            break;
                        default:
                            labelA = a.displayname.toUpperCase();
                            labelB = b.displayname.toUpperCase();
                            break;
                    }
                    var comparison = 0;
                    if (labelA > labelB) {
                        comparison = 1;
                    }
                    else if (labelA < labelB) {
                        comparison = -1;
                    }
                    return comparison;
                };
                ExportControlUtils.compareSelectOptionSet = function (a, b) {
                    var comparison = 0;
                    if (a.Label > b.Label) {
                        comparison = 1;
                    }
                    else if (a.Label < b.Label) {
                        comparison = -1;
                    }
                    return comparison;
                };
                ExportControlUtils.sortOnKeysAttributeFile = function (FieldRecordArrayObject) {
                    var temp = FieldRecordArrayObject;
                    temp = temp.sort(ExportControlUtils.compare);
                    return temp;
                };
                ExportControlUtils.sortOnLabel = function (EntitiesSelectorOptions) {
                    var temp = EntitiesSelectorOptions;
                    temp = temp.sort(ExportControlUtils.compareSelectOptionSet);
                    return temp;
                };
                return ExportControlUtils;
            }());
            ExportToExcel.ExportControlUtils = ExportControlUtils;
            var EntityFieldsSortByField;
            (function (EntityFieldsSortByField) {
                EntityFieldsSortByField[EntityFieldsSortByField["DisplayName"] = 0] = "DisplayName";
                EntityFieldsSortByField[EntityFieldsSortByField["Name"] = 1] = "Name";
                EntityFieldsSortByField[EntityFieldsSortByField["DataType"] = 2] = "DataType";
            })(EntityFieldsSortByField = ExportToExcel.EntityFieldsSortByField || (ExportToExcel.EntityFieldsSortByField = {}));
        })(ExportToExcel = AppCommon.ExportToExcel || (AppCommon.ExportToExcel = {}));
    })(AppCommon = MscrmControls.AppCommon || (MscrmControls.AppCommon = {}));
})(MscrmControls || (MscrmControls = {}));
/**
 * Module having utilities.
 */
var MscrmControls;
(function (MscrmControls) {
    var AppCommon;
    (function (AppCommon) {
        var ExportToExcel;
        (function (ExportToExcel) {
            "use strict";
            /**
             * String Utilities
             */
            var StringUtilities = (function () {
                function StringUtilities() {
                }
                Object.defineProperty(StringUtilities, "EmptyString", {
                    /**
                     * Empty string
                     */
                    get: function () {
                        return "";
                    },
                    enumerable: true,
                    configurable: true
                });
                /**
                * Is True string
                */
                StringUtilities.IsTrue = function (value) {
                    var returnValue = false;
                    try {
                        returnValue = eval(value.toLowerCase());
                    }
                    catch (e) {
                        returnValue = this.IsYes(value);
                    }
                    return returnValue;
                };
                /**
                * Is Yes
                */
                StringUtilities.IsYes = function (value) {
                    return value.toLowerCase() === "yes" || value.toLowerCase() === "y";
                };
                Object.defineProperty(StringUtilities, "WhiteSpace", {
                    /**
                     * Whitespace
                     */
                    get: function () {
                        return / /g;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(StringUtilities, "EnglishAndNumericCharacters", {
                    get: function () {
                        return /([a-z0-9A-Z]+)/g;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(StringUtilities, "ValidIdExpression", {
                    /**
                     * Valid ID expression
                     * As per MSDN, ID Specifies a unique identifier in ASCII. Valid values are a-z, A-Z, 0-9, and underscore (_). Spaces aren’t allowed
                     */
                    get: function () {
                        return /([a-z0-9A-Z_]+)/g;
                    },
                    enumerable: true,
                    configurable: true
                });
                /**
                 * Checks string is null,undefined or empty.
                 * @param str string for which we need to check.
                 */
                StringUtilities.IsNullOrEmpty = function (str) {
                    return str == null || str == undefined || str == "";
                };
                /**
                 * Formats the string with inputs provided as args.
                 * @param str Unformatted string.
                 * @param args Args to be replaced in the string.
                 */
                StringUtilities.Format = function (str, args) {
                    if (StringUtilities.IsNullOrEmpty(str)) {
                        return str;
                    }
                    return str.replace(/\{\{|\}\}|\{(\d+)\}/g, function (match, group) {
                        if (match == "{{") {
                            return "{";
                        }
                        if (match == "}}") {
                            return "}";
                        }
                        return args[group];
                    });
                };
                /**
                 * Checks if the main string ends with the sub string
                 * @param mainString The main string, in which sub string needs to be searched
                 * @param subString The smaller string that is to be searched
                 */
                StringUtilities.EndsWithString = function (mainString, subString) {
                    return mainString.indexOf(subString, mainString.length - subString.length) !== -1;
                };
                /**
                 * Checks if string contains only spaces, tabs or line breaks.
                 * @param str string for which we need to check
                */
                StringUtilities.IsBlankString = function (str) {
                    return str.replace(/\s/g, '').length == 0;
                };
                /**
                 * Escapes xml sensitive characters
                 * @param input string which needs to be escaped
                */
                StringUtilities.EscapeXml = function (input) {
                    return input.replace(/&/g, '&amp;')
                        .replace(/</g, '&lt;')
                        .replace(/>/g, '&gt;')
                        .replace(/"/g, '&quot;')
                        .replace(/'/g, '&apos;');
                };
                /**
                 * Escapes the special characters in Dom Id to be used as selector for angular.element
                 * @param input string which needs to be escaped
                */
                StringUtilities.EscapeDomId = function (input) {
                    return input.replace('.', '\\.');
                };
                StringUtilities.IsNull = function (value) {
                    return typeof (value) === 'undefined' || value == null;
                };
                StringUtilities.IsNotNull = function (value) {
                    return !StringUtilities.IsNull(value);
                };
                StringUtilities.IsNullOrEmptyString = function (str) {
                    return StringUtilities.IsNull(str) || (str == '');
                };
                StringUtilities.IsNullOrUndefined = function (value) {
                    return null == value || typeof (value) == 'undefined';
                };
                StringUtilities.IsNullOrWhiteSpace = function (value) {
                    return StringUtilities.IsNullOrEmptyString(value) || value.trim() == '';
                };
                StringUtilities.IsNullOrEmptyGuid = function (guid) {
                    return StringUtilities.IsNullOrEmptyString(guid) ||
                        StringUtilities.convertGuidToString(guid) === ExportToExcel.Constants.EmptyGuidFormatted;
                };
                StringUtilities.convertGuidToString = function (guid) {
                    if (guid != null) {
                        guid = guid.replace("{", "").replace("}", "");
                    }
                    return guid;
                };
                return StringUtilities;
            }());
            ExportToExcel.StringUtilities = StringUtilities;
        })(ExportToExcel = AppCommon.ExportToExcel || (AppCommon.ExportToExcel = {}));
    })(AppCommon = MscrmControls.AppCommon || (MscrmControls.AppCommon = {}));
})(MscrmControls || (MscrmControls = {}));
/**
 * Module having utilities.
 */
var MscrmControls;
(function (MscrmControls) {
    var AppCommon;
    (function (AppCommon) {
        var ExportToExcel;
        (function (ExportToExcel) {
            "use strict";
            /**
             * Generic javascript/typescript utilities
             */
            var XmlUtilities = (function () {
                function XmlUtilities() {
                }
                /**
                 * Checks object is null or undefined.
                 * @param obj Object for which we need to check
                 */
                XmlUtilities.IsNullOrUndefined = function (obj) {
                    return obj == null || obj == undefined;
                };
                /**
                 * Returns true if machine is a mac machine.
                 */
                XmlUtilities.IsMacMachine = function () {
                    return navigator.platform.indexOf("Mac") !== -1;
                };
                /**
                 * Checks nested object is null or undefined.
                 */
                XmlUtilities.IsNullOrUndefinedNested = function (objectNode, key) {
                    var isValid = key.split(".").every(function (x) {
                        if (typeof objectNode != "object" || objectNode == null || !(x in objectNode) || objectNode[x] == null)
                            return false;
                        objectNode = objectNode[x];
                        return true;
                    });
                    return isValid;
                };
                /**
                 * Parses the xml and generates to XMLDocument using JS runtime
                 * @param xml xml represented as string for which we need XMLDocument
                 */
                XmlUtilities.ParseXml = function (xml) {
                    if (window.hasOwnProperty("DOMParser")) {
                        var parser = new DOMParser();
                        return parser.parseFromString(xml, "text/xml");
                    }
                    else if (window.hasOwnProperty("ActiveXObject")) {
                        var xmlDoc = new window.ActiveXObject("Microsoft.XMLDOM"); // tslint:disable-line:no-any
                        xmlDoc.async = false;
                        xmlDoc.loadXML(xml);
                        return xmlDoc;
                    }
                    return null;
                };
                /**
                * To get value of an Attribute node
                * @param attributes collection of attributes
                * @param namedItem name of attribute
                */
                XmlUtilities.getNamedItemValue = function (attributes, namedItem) {
                    var attribute = attributes.getNamedItem(namedItem);
                    if (!ExportToExcel.StringUtilities.IsNullOrUndefined(attribute)) {
                        return attribute.value;
                    }
                    return null;
                };
                /**
                * Returns a  boolean value
                * @param attributes collection of attributes
                * @param namedItem name of attribute
                */
                XmlUtilities.getBooleanValue = function (attributes, namedItem) {
                    var value = XmlUtilities.getNamedItemValue(attributes, namedItem);
                    if (!ExportToExcel.StringUtilities.IsNullOrUndefined(value)) {
                        return ExportToExcel.StringUtilities.IsTrue(value);
                    }
                    return null;
                };
                /**
                * Returns a  numeric value
                * @param value value of attribute
                */
                XmlUtilities.getNumericValue = function (value) {
                    if (!ExportToExcel.StringUtilities.IsNullOrUndefined(value)) {
                        return parseInt(value);
                    }
                    return null;
                };
                XmlUtilities.convertGuidToString = function (guid) {
                    if (guid != null) {
                        guid = guid.replace("{", "").replace("}", "");
                    }
                    return guid;
                };
                return XmlUtilities;
            }());
            ExportToExcel.XmlUtilities = XmlUtilities;
        })(ExportToExcel = AppCommon.ExportToExcel || (AppCommon.ExportToExcel = {}));
    })(AppCommon = MscrmControls.AppCommon || (MscrmControls.AppCommon = {}));
})(MscrmControls || (MscrmControls = {}));
var MscrmControls;
(function (MscrmControls) {
    var AppCommon;
    (function (AppCommon) {
        var ExportToExcel;
        (function (ExportToExcel) {
            'use strict';
            var EntityDetails = (function () {
                function EntityDetails() {
                    this.selectedAttributes = [];
                }
                return EntityDetails;
            }());
            ExportToExcel.EntityDetails = EntityDetails;
            var EntityFields = (function () {
                function EntityFields() {
                    this.entityAttributesList = [];
                }
                return EntityFields;
            }());
            ExportToExcel.EntityFields = EntityFields;
        })(ExportToExcel = AppCommon.ExportToExcel || (AppCommon.ExportToExcel = {}));
    })(AppCommon = MscrmControls.AppCommon || (MscrmControls.AppCommon = {}));
})(MscrmControls || (MscrmControls = {}));
var MscrmControls;
(function (MscrmControls) {
    var AppCommon;
    (function (AppCommon) {
        var ExportToExcel;
        (function (ExportToExcel) {
            'use strict';
            var FieldRecord = (function () {
                function FieldRecord() {
                }
                return FieldRecord;
            }());
            ExportToExcel.FieldRecord = FieldRecord;
            //export class FieldDisplayName {
            //    public Description: string;
            //    public LanguageCode: string;
            //}
            //export class FieldOptionLabel {
            //    public Description: string;
            //    public LanguageCode: string;
            //}
            //export class FieldOption {
            //    public Value: string;
            //    public Labels :FieldOptionLabel []
            //}
            //public name: string;
            //    public requiredlevel: string;
            //    public imemode: string;
            //    public format: string;
            //    public displaynames: FieldDisplayName[];
            //    public FieldOptions: FieldOption[];
            //    public isSortEnabled: boolean;
            //    public maxlength: number;
            //    public minvalue: number;
            //    public maxvalue: number;
            //    public accuracy: number;
            //    public lookupstyle: string
            //    public lookupbrowse: string;
            //    public lookuptypes: string;
            //    public defaultvalue: number;
            //    public sourcetype: number;
            //    public isglobalfilterenabled: boolean;
            //    public validforupdateapi: boolean;
            //    public validforreadapi: boolean;
            //    public requiredforplatform: boolean;
            //    public requiredforgrid: boolean;
            //    public requiredforform: boolean;
            //    public validforadvancedfind: boolean;
            //    public validforcreateapi: boolean;
            //    public validforgrid: boolean;
            //    public validforform: boolean;
            //    public objecttypecode: number;
            //    public customfield: boolean;
            //    public isnonsortablecalculatedfield: boolean;
            //    public datatype: number;
        })(ExportToExcel = AppCommon.ExportToExcel || (AppCommon.ExportToExcel = {}));
    })(AppCommon = MscrmControls.AppCommon || (MscrmControls.AppCommon = {}));
})(MscrmControls || (MscrmControls = {}));
/**
* @license Copyright (c) Microsoft Corporation.  All rights reserved.
*/
var MscrmControls;
(function (MscrmControls) {
    var AppCommon;
    (function (AppCommon) {
        var ExportToExcel;
        (function (ExportToExcel) {
            'use strict';
            /**
            * The name of this event which will be reported to Telemetry pipeline
            */
            var ExcelControlInit = "ExcelControlInit";
            var ExcelControlFailure = "ExcelControlFailure";
            var ExcelControlInitEvent = (function () {
                function ExcelControlInitEvent(entityTypeCode, exportType) {
                    this.eventName = ExcelControlInit;
                    this.eventParameters = [
                        { name: "entityTypeCode", value: entityTypeCode },
                        { name: "exportType", value: exportType },
                    ];
                }
                return ExcelControlInitEvent;
            }());
            ExportToExcel.ExcelControlInitEvent = ExcelControlInitEvent;
            var ExcelControlFailureEvent = (function () {
                function ExcelControlFailureEvent(entityType, exportType, message) {
                    this.eventName = ExcelControlFailure;
                    this.eventParameters = [
                        { name: "entityType", value: entityType },
                        { name: "exportType", value: exportType },
                        { name: "message", value: message },
                    ];
                }
                return ExcelControlFailureEvent;
            }());
            ExportToExcel.ExcelControlFailureEvent = ExcelControlFailureEvent;
        })(ExportToExcel = AppCommon.ExportToExcel || (AppCommon.ExportToExcel = {}));
    })(AppCommon = MscrmControls.AppCommon || (MscrmControls.AppCommon = {}));
})(MscrmControls || (MscrmControls = {}));
//# sourceMappingURL=ExportToExcelControl.js.map